// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AI32SSA/driver/16ai32ssa.h $
// $Rev: 50232 $
// $Date: 2022-02-09 16:20:21 -0600 (Wed, 09 Feb 2022) $

// 16AI32SSA: Device Driver: header file

#ifndef	__16AI32SSA_H__
#define	__16AI32SSA_H__

#include "gsc_common.h"
#include "gsc_pci9056.h"



// macros *********************************************************************

#define	AI32SSA_BASE_NAME					"16ai32ssa"

// IOCTL command codes
#define	AI32SSA_IOCTL_REG_READ				OS_IOCTL_RW( 0, 12, gsc_reg_t)
#define	AI32SSA_IOCTL_REG_WRITE				OS_IOCTL_W ( 1, 12, gsc_reg_t)
#define	AI32SSA_IOCTL_REG_MOD				OS_IOCTL_W ( 2, 12, gsc_reg_t)
#define	AI32SSA_IOCTL_QUERY					OS_IOCTL_RW( 3,  4, s32)
#define	AI32SSA_IOCTL_INITIALIZE			OS_IOCTL   ( 4)
#define	AI32SSA_IOCTL_AUTO_CALIBRATE		OS_IOCTL   ( 5)
#define	AI32SSA_IOCTL_AUTO_CAL_STS			OS_IOCTL_R ( 6,  4, s32)
#define	AI32SSA_IOCTL_AIN_BUF_CLEAR			OS_IOCTL   ( 7)
#define	AI32SSA_IOCTL_AIN_BUF_LEVEL			OS_IOCTL_R ( 8,  4, s32)
#define	AI32SSA_IOCTL_AIN_BUF_OVERFLOW		OS_IOCTL_RW( 9,  4, s32)
#define	AI32SSA_IOCTL_AIN_BUF_THR_LVL		OS_IOCTL_RW(10,  4, s32)
#define	AI32SSA_IOCTL_AIN_BUF_THR_STS		OS_IOCTL_R (11,  4, s32)
#define	AI32SSA_IOCTL_AIN_BUF_UNDERFLOW		OS_IOCTL_RW(12,  4, s32)
#define	AI32SSA_IOCTL_AIN_MODE				OS_IOCTL_RW(13,  4, s32)
#define	AI32SSA_IOCTL_AIN_RANGE				OS_IOCTL_RW(14,  4, s32)
#define	AI32SSA_IOCTL_AUX_0_MODE			OS_IOCTL_RW(15,  4, s32)
#define	AI32SSA_IOCTL_AUX_1_MODE			OS_IOCTL_RW(16,  4, s32)
#define	AI32SSA_IOCTL_AUX_IN_POL			OS_IOCTL_RW(17,  4, s32)
#define	AI32SSA_IOCTL_AUX_NOISE				OS_IOCTL_RW(18,  4, s32)
#define	AI32SSA_IOCTL_AUX_OUT_POL			OS_IOCTL_RW(19,  4, s32)
#define	AI32SSA_IOCTL_AUX_SYNC_ENABLE		OS_IOCTL_RW(20,  4, s32)
#define	AI32SSA_IOCTL_BURST_BUSY			OS_IOCTL_R (21,  4, s32)
#define	AI32SSA_IOCTL_BURST_SIZE			OS_IOCTL_RW(22,  4, s32)
#define	AI32SSA_IOCTL_BURST_SRC				OS_IOCTL_RW(23,  4, s32)
#define	AI32SSA_IOCTL_CHAN_ACTIVE			OS_IOCTL_RW(24,  4, s32)
#define	AI32SSA_IOCTL_CHAN_FIRST			OS_IOCTL_RW(25,  4, s32)
#define	AI32SSA_IOCTL_CHAN_LAST				OS_IOCTL_RW(26,  4, s32)
#define	AI32SSA_IOCTL_CHAN_SINGLE			OS_IOCTL_RW(27,  4, s32)
#define	AI32SSA_IOCTL_DATA_FORMAT			OS_IOCTL_RW(28,  4, s32)
#define	AI32SSA_IOCTL_DATA_PACKING			OS_IOCTL_RW(29,  4, s32)
#define	AI32SSA_IOCTL_DMDMA_MODE			OS_IOCTL_RW(30,  4, s32)
#define	AI32SSA_IOCTL_INPUT_SYNC			OS_IOCTL   (31)
#define	AI32SSA_IOCTL_IRQ0_SEL				OS_IOCTL_RW(32,  4, s32)
#define	AI32SSA_IOCTL_IRQ1_SEL				OS_IOCTL_RW(33,  4, s32)
#define	AI32SSA_IOCTL_RAG_ENABLE			OS_IOCTL_RW(34,  4, s32)
#define	AI32SSA_IOCTL_RAG_NRATE				OS_IOCTL_RW(35,  4, s32)
#define	AI32SSA_IOCTL_RBG_CLK_SRC			OS_IOCTL_RW(36,  4, s32)
#define	AI32SSA_IOCTL_RBG_ENABLE			OS_IOCTL_RW(37,  4, s32)
#define	AI32SSA_IOCTL_RBG_NRATE				OS_IOCTL_RW(38,  4, s32)
#define	AI32SSA_IOCTL_RX_IO_ABORT			OS_IOCTL_R (39,  4, s32)
#define	AI32SSA_IOCTL_RX_IO_MODE			OS_IOCTL_RW(40,  4, s32)
#define	AI32SSA_IOCTL_RX_IO_OVERFLOW		OS_IOCTL_RW(41,  4, s32)
#define	AI32SSA_IOCTL_RX_IO_TIMEOUT			OS_IOCTL_RW(42,  4, s32)
#define	AI32SSA_IOCTL_RX_IO_UNDERFLOW		OS_IOCTL_RW(43,  4, s32)
#define	AI32SSA_IOCTL_SAMP_CLK_SRC			OS_IOCTL_RW(44,  4, s32)
#define	AI32SSA_IOCTL_SCAN_MARKER			OS_IOCTL_RW(45,  4, s32)
#define	AI32SSA_IOCTL_SCAN_MARKER_VAL		OS_IOCTL_RW(46,  4, u32)
#define	AI32SSA_IOCTL_WAIT_CANCEL			OS_IOCTL_RW(47, 28, gsc_wait_t)
#define	AI32SSA_IOCTL_WAIT_EVENT			OS_IOCTL_RW(48, 28, gsc_wait_t)
#define	AI32SSA_IOCTL_WAIT_STATUS			OS_IOCTL_RW(49, 28, gsc_wait_t)

//*****************************************************************************
// AI32SSA_IOCTL_REG_READ
// AI32SSA_IOCTL_REG_WRITE
// AI32SSA_IOCTL_REG_MOD
//
#define	AI32SSA_REG_ENCODE(s,o)				GSC_REG_ENCODE(GSC_REG_TYPE_BAR2,(s),(o))
// Parameter:	gsc_reg_t*
#define	AI32SSA_GSC_BCTLR					AI32SSA_REG_ENCODE(4,0x00)// Board Control Register
#define	AI32SSA_GSC_ICR						AI32SSA_REG_ENCODE(4,0x04)// Interrupt Control Register
#define	AI32SSA_GSC_IDBR					AI32SSA_REG_ENCODE(4,0x08)// Input Data Buffer Register
#define	AI32SSA_GSC_IBCR					AI32SSA_REG_ENCODE(4,0x0C)// Input Buffer Control Register
#define	AI32SSA_GSC_RAGR					AI32SSA_REG_ENCODE(4,0x10)// Rate-A Generator Register
#define	AI32SSA_GSC_RBGR					AI32SSA_REG_ENCODE(4,0x14)// Rate-B Generator Register
#define	AI32SSA_GSC_BUFSR					AI32SSA_REG_ENCODE(4,0x18)// Buffer Size Register
#define	AI32SSA_GSC_BSIZR					AI32SSA_REG_ENCODE(4,0x1C)// Burst Size Register
#define	AI32SSA_GSC_SSCR					AI32SSA_REG_ENCODE(4,0x20)// Scan & Sync Control Register
#define	AI32SSA_GSC_ACAR					AI32SSA_REG_ENCODE(4,0x24)// Active Channel Assignment Register
#define	AI32SSA_GSC_BCFGR					AI32SSA_REG_ENCODE(4,0x28)// Board Configuration Register
#define	AI32SSA_GSC_ACVR					AI32SSA_REG_ENCODE(4,0x2C)// Auto Cal Values Register
#define	AI32SSA_GSC_ARWR					AI32SSA_REG_ENCODE(4,0x30)// Auxiliary R/W Register
#define	AI32SSA_GSC_ASIOCR					AI32SSA_REG_ENCODE(4,0x34)// Auxiliary Sync I/O Control Register
#define	AI32SSA_GSC_SMUWR					AI32SSA_REG_ENCODE(4,0x38)// Scan Marker Upper Word Register
#define	AI32SSA_GSC_SMLWR					AI32SSA_REG_ENCODE(4,0x3C)// Scan Marker Lower Word Register

//*****************************************************************************
// AI32SSA_IOCTL_QUERY
//
//	Parameter:	s32
//		Pass in a value from the list below.
//		The value returned is the answer to the query.

typedef enum
{
	AI32SSA_QUERY_AUTO_CAL_MS,		// Max auto-cal period in ms.
	AI32SSA_QUERY_CHANNEL_MAX,		// Maximum number of channels supported.
	AI32SSA_QUERY_CHANNEL_QTY,		// The number of A/D channels.
	AI32SSA_QUERY_COUNT,			// How many query options are supported?
	AI32SSA_QUERY_DEVICE_TYPE,		// Value from gsc_dev_type_t
	AI32SSA_QUERY_FIFO_SIZE,		// FIFO depth in 32-bit samples
	AI32SSA_QUERY_FSAMP_MAX,		// The maximum sample rate per channel.
	AI32SSA_QUERY_FSAMP_MIN,		// The minimum sample rate per channel.
	AI32SSA_QUERY_INIT_MS,			// Max initialize period in ms.
	AI32SSA_QUERY_MASTER_CLOCK,		// Master clock frequency
	AI32SSA_QUERY_NRATE_MASK,		// Mask of valid Nrate bits.
	AI32SSA_QUERY_NRATE_MAX,		// Maximum rate generator Nrate value.
	AI32SSA_QUERY_NRATE_MIN,		// Maximum rate generator Nrate value.
	AI32SSA_QUERY_RATE_GEN_QTY,		// Number of Rate Generatorts.
	AI32SSA_QUERY_V_RANGE_HIGH,		// Is the high voltage range present.
	AI32SSA_IOCTL_QUERY_LAST		// This is the last entry.
} ai32ssa_query_t;

#define	AI32SSA_IOCTL_QUERY_ERROR			(-1)

//*****************************************************************************
// AI32SSA_IOCTL_INITIALIZE					BCTLR D15
//
//	Parameter:	None

//*****************************************************************************
// AI32SSA_IOCTL_AUTO_CALIBRATE				BCTLR D13
//
//	Parameter:	None

//*****************************************************************************
// AI32SSA_IOCTL_AUTO_CAL_STS				BCTLR D13, D14
//
//	Parameter:	s32*
//		The value returned is one of the following.
#define	AI32SSA_AUTO_CAL_STS_ACTIVE			0
#define	AI32SSA_AUTO_CAL_STS_FAIL			1
#define	AI32SSA_AUTO_CAL_STS_PASS			2

//*****************************************************************************
// AI32SSA_IOCTL_AIN_BUF_CLEAR				IBCR D18
//
//	Parameter:	None.

//*****************************************************************************
// AI32SSA_IOCTL_AIN_BUF_LEVEL				BUFSR D0-D18
//
//	Parameter:	s32*
//		The value returned is the current input buffer fill level.

//*****************************************************************************
// AI32SSA_IOCTL_AIN_BUF_OVERFLOW			BCTLR D17
//
//	Parameter:	s32*
//		Pass in any valid option below, or
//		-1 to read the current setting.
#define	AI32SSA_AIN_BUF_OVERFLOW_CLEAR		0
#define	AI32SSA_AIN_BUF_OVERFLOW_CHECK		(-1)

// For queries the following values are returned.
#define	AI32SSA_AIN_BUF_OVERFLOW_NO			0
#define	AI32SSA_AIN_BUF_OVERFLOW_YES		1

//*****************************************************************************
// AI32SSA_IOCTL_AIN_BUF_THR_LVL			IBCR D0-D17
//
//	Parameter:	s32*
//		Pass in any valid value from 0x0 to 0x3FFFF, or
//		-1 to read the current setting.

//*****************************************************************************
// AI32SSA_IOCTL_AIN_BUF_THR_STS			IBCR D19
//
//	Parameter:	s32*
//		The value returned is one of the following.
#define	AI32SSA_AIN_BUF_THR_STS_CLEAR		0
#define	AI32SSA_AIN_BUF_THR_STS_SET			1

//*****************************************************************************
// AI32SSA_IOCTL_AIN_BUF_UNDERFLOW			BCTLR D16
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AI32SSA_AIN_BUF_UNDERFLOW_CLEAR		0
#define	AI32SSA_AIN_BUF_UNDERFLOW_CHECK		(-1)

// For queries the following values are returned.
#define	AI32SSA_AIN_BUF_UNDERFLOW_NO		0
#define	AI32SSA_AIN_BUF_UNDERFLOW_YES		1

//*****************************************************************************
// AI32SSA_IOCTL_AIN_MODE					BCTLR D0-D2
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AI32SSA_AIN_MODE_SYSTEM				0	// Differential
#define	AI32SSA_AIN_MODE_ZERO				2	// Zero test
#define	AI32SSA_AIN_MODE_VREF				3	// Vref test

//*****************************************************************************
// AI32SSA_IOCTL_AIN_RANGE					BCTLR D4-D5
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AI32SSA_AIN_RANGE_0_625V			0	// +- 0.625 volts
#define	AI32SSA_AIN_RANGE_1_25V				1	// +- 1.25 volts
#define	AI32SSA_AIN_RANGE_2_5V				2	// +- 2.5 volts
#define	AI32SSA_AIN_RANGE_5V				3	// +- 5 volts
#define	AI32SSA_AIN_RANGE_10V				4	// +- 10 volts

//*****************************************************************************
// AI32SSA_IOCTL_AUX_0_MODE					ASIOCR D0-D1
// AI32SSA_IOCTL_AUX_1_MODE					ASIOCR D2-D3
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AI32SSA_AUX_MODE_DISABLE			0
#define	AI32SSA_AUX_MODE_INPUT				1
#define	AI32SSA_AUX_MODE_OUTPUT				2

//*****************************************************************************
// AI32SSA_IOCTL_AUX_IN_POL					ASIOCR D8
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AI32SSA_AUX_IN_POL_LO_2_HI			0
#define	AI32SSA_AUX_IN_POL_HI_2_LO			1

//*****************************************************************************
// AI32SSA_IOCTL_AUX_NOISE					ASIOCR D10
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AI32SSA_AUX_NOISE_LOW				0
#define	AI32SSA_AUX_NOISE_HIGH				1

//*****************************************************************************
// AI32SSA_IOCTL_AUX_OUT_POL				ASIOCR D9
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AI32SSA_AUX_OUT_POL_HI_PULSE		0
#define	AI32SSA_AUX_OUT_POL_LOW_PULSE		1

//*****************************************************************************
// AI32SSA_IOCTL_AUX_SYNC_ENABLE			BCTLR D7
//
//	Parameter:	s32*
//		Pass in any valid option below, or
//		-1 to read the current setting.
#define	AI32SSA_AUX_SYNC_ENABLE_NO			0
#define	AI32SSA_AUX_SYNC_ENABLE_YES			1

//*****************************************************************************
// AI32SSA_IOCTL_BURST_BUSY					SSCR D7
//
//	Parameter:	s32*
//		The value returned is one of the following.
#define	AI32SSA_BURST_BUSY_IDLE				0
#define	AI32SSA_BURST_BUSY_ACTIVE			1

//*****************************************************************************
// AI32SSA_IOCTL_BURST_SIZE					BSIZR D0-D19
//
//	Parameter:	s32*
//		Pass in any value between the below inclusive limits, or
//		-1 to read the current setting.
//		At this time the valid range is 0-0xFFFFF.

//*****************************************************************************
// AI32SSA_IOCTL_BURST_SRC					SSCR D8-D9
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AI32SSA_BURST_SRC_DISABLE			0
#define	AI32SSA_BURST_SRC_RBG				1
#define	AI32SSA_BURST_SRC_EXT				2
#define	AI32SSA_BURST_SRC_BCR				3

//*****************************************************************************
// AI32SSA_IOCTL_CHAN_ACTIVE				SSCR D0-D2
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AI32SSA_CHAN_ACTIVE_SINGLE			0
#define	AI32SSA_CHAN_ACTIVE_0_1				1
#define	AI32SSA_CHAN_ACTIVE_0_3				2
#define	AI32SSA_CHAN_ACTIVE_0_7				3
#define	AI32SSA_CHAN_ACTIVE_0_15			4
#define	AI32SSA_CHAN_ACTIVE_0_3X			5
#define	AI32SSA_CHAN_ACTIVE_RANGE			7

//*****************************************************************************
// AI32SSA_IOCTL_CHAN_FIRST					ACAR D0-D7
//
//	Parameter:	s32*
//		Pass in any valid value from zero to the index of the last channel,
//		but not more than the LAST channel selection, or
//		-1 to read the current setting.

//*****************************************************************************
// AI32SSA_IOCTL_CHAN_LAST					ACAR D8-D15
//
//	Parameter:	s32*
//		Pass in any valid value from zero to the index of the last channel,
//		but not less than the FIRST channel selection, or
//		-1 to read the current setting.

//*****************************************************************************
// AI32SSA_IOCTL_CHAN_SINGLE				SSCR D12-D17
//
//	Parameter:	s32*
//		Pass in any valid value from zero to the index of the last channel, or
//		-1 to read the current setting.

//*****************************************************************************
// AI32SSA_IOCTL_DATA_FORMAT				BCTLR D6
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AI32SSA_DATA_FORMAT_2S_COMP			0	// Twos Compliment
#define	AI32SSA_DATA_FORMAT_OFF_BIN			1	// Offset Binary

//*****************************************************************************
// AI32SSA_IOCTL_DATA_PACKING				BCTLR D18
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AI32SSA_DATA_PACKING_DISABLE		0
#define	AI32SSA_DATA_PACKING_ENABLE			1

//*****************************************************************************
// AI32SSA_IOCTL_DMDMA_MODE					BCTLR D19
//
//	Parameter:	s32*
//		Pass in any valid option below, or
//		-1 to read the current setting.
#define	AI32SSA_DMDMA_MODE_THRESHOLD		0
#define	AI32SSA_DMDMA_MODE_AUTONOMOUS		1

//*****************************************************************************
// AI32SSA_IOCTL_INPUT_SYNC					BCTLR D12
//
//	Parameter:	None.

//*****************************************************************************
// AI32SSA_IOCTL_IRQ0_SEL					ICR D0-D2
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AI32SSA_IRQ0_INIT_DONE				0
#define	AI32SSA_IRQ0_AUTO_CAL_DONE			1
#define	AI32SSA_IRQ0_SYNC_START				2
#define	AI32SSA_IRQ0_SYNC_DONE				3
#define	AI32SSA_IRQ0_BURST_START			4
#define	AI32SSA_IRQ0_BURST_DONE				5

//*****************************************************************************
// AI32SSA_IOCTL_IRQ1_SEL					ICR D4-D6
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AI32SSA_IRQ1_NONE					0
#define	AI32SSA_IRQ1_IN_BUF_THR_L2H			1
#define	AI32SSA_IRQ1_IN_BUF_THR_H2L			2
#define	AI32SSA_IRQ1_IN_BUF_OVR_UNDR		3

//*****************************************************************************
// AI32SSA_IOCTL_RAG_ENABLE					RAGR D16
// AI32SSA_IOCTL_RBG_ENABLE					RBGR D16
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AI32SSA_GEN_ENABLE_NO				1
#define	AI32SSA_GEN_ENABLE_YES				0

//*****************************************************************************
// AI32SSA_IOCTL_RAG_NRATE					RAGR D0-D15
// AI32SSA_IOCTL_RBG_NRATE					RBGR D0-D15
//	Parameter:	s32*
//		Pass in any value between the below inclusive limits, or
//		-1 to read the current setting.
//		At this time the valid range is 200-0xFFFF.

//*****************************************************************************
// AI32SSA_IOCTL_RBG_CLK_SRC				SSCR D10
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AI32SSA_RBG_CLK_SRC_MASTER			0
#define	AI32SSA_RBG_CLK_SRC_RAG				1

//*****************************************************************************
// AI32SSA_IOCTL_RX_IO_ABORT
//
//	Parameter:	s32*
//		The returned value is one of the below options.
#define	AI32SSA_IO_ABORT_NO					0
#define	AI32SSA_IO_ABORT_YES				1

//*****************************************************************************
// AI32SSA_IOCTL_RX_IO_MODE
//
//	Parameter:	s32*
//		Pass in any of the gsc_io_mode_t options, or
//		-1 to read the current setting.
#define	AI32SSA_IO_MODE_DEFAULT				GSC_IO_MODE_PIO
// GSC_IO_MODE_PIO
// GSC_IO_MODE_BMDMA
// GSC_IO_MODE_DMDMA

//*****************************************************************************
// AI32SSA_IOCTL_RX_IO_OVERFLOW
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AI32SSA_IO_OVERFLOW_DEFAULT			AI32SSA_IO_OVERFLOW_CHECK
#define	AI32SSA_IO_OVERFLOW_IGNORE			0
#define	AI32SSA_IO_OVERFLOW_CHECK			1

//*****************************************************************************
// AI32SSA_IOCTL_RX_IO_TIMEOUT (in seconds)
//
//	Parameter:	s32*
//		Pass in any value from the minimim to the maximim, the infinite option,
//		or -1 to read the current setting. The value -1 is returned if the
//		feature is not supported, but this should never happen.
#define	AI32SSA_IO_TIMEOUT_DEFAULT			10
#define	AI32SSA_IO_TIMEOUT_NO_SLEEP			0
#define	AI32SSA_IO_TIMEOUT_MIN				0
#define	AI32SSA_IO_TIMEOUT_MAX				GSC_IO_TIMEOUT_MAX
#define	AI32SSA_IO_TIMEOUT_INFINITE			GSC_IO_TIMEOUT_INFINITE

//*****************************************************************************
// AI32SSA_IOCTL_RX_IO_UNDERFLOW
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AI32SSA_IO_UNDERFLOW_DEFAULT		AI32SSA_IO_UNDERFLOW_CHECK
#define	AI32SSA_IO_UNDERFLOW_IGNORE			0
#define	AI32SSA_IO_UNDERFLOW_CHECK			1

//*****************************************************************************
// AI32SSA_IOCTL_SAMP_CLK_SRC				SSCR D3-D4
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AI32SSA_SAMP_CLK_SRC_RAG			0
#define	AI32SSA_SAMP_CLK_SRC_RBG			1
#define	AI32SSA_SAMP_CLK_SRC_EXT			2
#define	AI32SSA_SAMP_CLK_SRC_BCR			3

//*****************************************************************************
// AI32SSA_IOCTL_SCAN_MARKER				BCTLR D11
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AI32SSA_SCAN_MARKER_DISABLE			1
#define	AI32SSA_SCAN_MARKER_ENABLE			0

//*****************************************************************************
// AI32SSA_IOCTL_SCAN_MARKER_VAL			SMUWR D0-D15, SMLWR D0-D15
//
//	Parameter:	u32*
//		Pass in any value between the range 0 to 0xFFFFFFFF, or
//		-1 to read the current setting.

//*****************************************************************************
// AI32SSA_IOCTL_WAIT_CANCEL				fields need not be valid
// AI32SSA_IOCTL_WAIT_EVENT					all fields must be valid
// AI32SSA_IOCTL_WAIT_STATUS				fields need not be valid
//
//	Parameter:	gsc_wait_t*
// gsc_wait_t.flags - see gsc_common.h
// gsc_wait_t.main - see gsc_common.h
// gsc_wait_t.gsc
#define	AI32SSA_WAIT_GSC_INIT_DONE			0x0001
#define	AI32SSA_WAIT_GSC_AUTO_CAL_DONE		0x0002
#define	AI32SSA_WAIT_GSC_SYNC_START			0x0004
#define	AI32SSA_WAIT_GSC_SYNC_DONE			0x0008
#define	AI32SSA_WAIT_GSC_BURST_START		0x0010
#define	AI32SSA_WAIT_GSC_BURST_DONE			0x0020
#define	AI32SSA_WAIT_GSC_IN_BUF_THR_L2H		0x0040
#define	AI32SSA_WAIT_GSC_IN_BUF_THR_H2L		0x0080
#define	AI32SSA_WAIT_GSC_IN_BUF_OVR_UNDR	0x0100
#define	AI32SSA_WAIT_GSC_ALL				0x01FF
// gsc_wait_t.alt flags
#define	AI32SSA_WAIT_ALT_ALL				0x0000
// gsc_wait_t.io
#define	AI32SSA_WAIT_IO_RX_ABORT			0x0001
#define	AI32SSA_WAIT_IO_RX_DONE				0x0002
#define	AI32SSA_WAIT_IO_RX_ERROR			0x0004
#define	AI32SSA_WAIT_IO_RX_TIMEOUT			0x0008

#define	AI32SSA_WAIT_IO_ALL					0x000F



#endif
