// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AI32SSA/samples/rxrate/perform.c $
// $Rev: 50224 $
// $Date: 2022-02-09 15:54:54 -0600 (Wed, 09 Feb 2022) $

// 16AI32SSA: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_1M	(1024L * 1024L)



// variables ******************************************************************

static u32	_rx[_1M];



//*****************************************************************************
static int _rate_test(const args_t* args, int seconds, long get)
{
	struct timeval	begin;
	int				errs	= 0;
	long			got;
	long long		limit	= (long long) args->rx_mb * 1000000L;
	struct timeval	minimum;
	struct timeval	now;
	long long		sps;
	long long		total	= 0;
	long			us		= 1000000L * seconds;

	gsc_label("Reading");

	gettimeofday(&begin, NULL);
	minimum.tv_sec	= begin.tv_sec + seconds;
	minimum.tv_usec	= begin.tv_usec;

	for (;;)
	{
		got	= ai32ssa_read(args->fd, _rx, get);

		if (got < 0)
		{
			errs++;
			printf("FAIL <--- (ai32ssa_read");
			printf(", total so far %lld", total);
			printf(", requested %ld bytes", (long) get);
			printf(", returned %ld)\n", got);
			break;
		}

		gettimeofday(&now, NULL);
		total	+= got;

		if ((total >= limit) && (now.tv_sec > minimum.tv_sec))
		{
			us	= (now.tv_sec - begin.tv_sec) * 1000000L
				+ (now.tv_usec - begin.tv_usec);
			sps	= (total * 10000000L / 4 / us + 5) / 10;

			printf(	"PASS  (");
			gsc_label_long_comma(total);
			printf(" Bytes, %ld.%06ld Seconds, ", us / 1000000, us % 1000000);
			gsc_label_long_comma(sps);
			printf(" S/S)\n");
			break;
		}
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int		errs	= 0;
	long	get		= sizeof(_rx);
	int		seconds	= 5;

	errs	+= ai32ssa_config_ain		(args->fd, -1, 1, 200);
	errs	+= ai32ssa_rx_io_mode		(args->fd, -1, 1, args->io_mode, NULL);
	errs	+= ai32ssa_rx_io_overflow	(args->fd, -1, 1, AI32SSA_IO_OVERFLOW_IGNORE, NULL);
	errs	+= ai32ssa_rx_io_underflow	(args->fd, -1, 1, AI32SSA_IO_UNDERFLOW_IGNORE, NULL);

	gsc_label ("Read Rate");
	printf("(%d second minimum, %ld MB minimum)\n", seconds, (long) args->rx_mb);
	gsc_label_level_inc();

	errs	+= ai32ssa_ain_buf_clear	(args->fd, -1, 1);
	errs	+= ai32ssa_ain_buf_overflow	(args->fd, -1, 1, AI32SSA_AIN_BUF_OVERFLOW_CLEAR, NULL);
	errs	+= ai32ssa_ain_buf_underflow(args->fd, -1, 1, AI32SSA_AIN_BUF_UNDERFLOW_CLEAR, NULL);

	errs	+= _rate_test(args, seconds, get);

	errs	+= ai32ssa_ain_buf_overflow	(args->fd, -1, 1, AI32SSA_AIN_BUF_OVERFLOW_CHECK, NULL);
	errs	+= ai32ssa_ain_buf_underflow(args->fd, -1, 1, AI32SSA_AIN_BUF_UNDERFLOW_CHECK, NULL);

	gsc_label_level_dec();
	return(errs);
}



