// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AI32SSA/samples/savedata/perform.c $
// $Rev: 51918 $
// $Date: 2022-10-20 17:30:59 -0500 (Thu, 20 Oct 2022) $

// 16AI32SSA: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_1M		(1024L * 1024L)



// variables ******************************************************************

static	u32	_buffer[_1M];



//*****************************************************************************
static int _channels(const args_t* args, s32* chans)
{
	s32		active;
	char	buf[64];
	int		errs	= 0;
	s32		first	= -1;
	s32		last	= -1;
	s32		qty		= 32;
	int		ret;
	s32		single	= -1;

	gsc_label("Input Channels");

	active	= -1;
	ret		= ai32ssa_ioctl(args->fd, AI32SSA_IOCTL_CHAN_ACTIVE,	&active	);
	errs	+= ret ? 1 : 0;
	ret		= ai32ssa_ioctl(args->fd, AI32SSA_IOCTL_CHAN_FIRST,		&first	);
	errs	+= ret ? 1 : 0;
	ret		= ai32ssa_ioctl(args->fd, AI32SSA_IOCTL_CHAN_LAST,		&last	);
	errs	+= ret ? 1 : 0;
	ret		= ai32ssa_ioctl(args->fd, AI32SSA_IOCTL_CHAN_SINGLE,	&single	);
	errs	+= ret ? 1 : 0;
	ret		= ai32ssa_ioctl(args->fd, AI32SSA_QUERY_CHANNEL_QTY,	&qty	);
	errs	+= ret ? 1 : 0;

	if (errs == 0)
	{
		switch (active)
		{
			default:

				errs++;
				printf("FAIL <---  (invalid ACTIVE selection: %ld)\n", (long) active);
				break;

			case AI32SSA_CHAN_ACTIVE_SINGLE:

				chans[0]	= 1;
				sprintf(buf, "# %ld", (long) single);
				break;

			case AI32SSA_CHAN_ACTIVE_0_1:

				chans[0]	= 2;
				strcpy(buf, "#s 0-1");
				break;

			case AI32SSA_CHAN_ACTIVE_0_3:

				chans[0]	= 4;
				strcpy(buf, "#s 0-3");
				break;

			case AI32SSA_CHAN_ACTIVE_0_7:

				chans[0]	= 8;
				strcpy(buf, "#s 0-7");
				break;

			case AI32SSA_CHAN_ACTIVE_0_15:

				chans[0]	= 16;
				strcpy(buf, "#s 0-15");
				break;

			case AI32SSA_CHAN_ACTIVE_0_3X:

				chans[0]	= qty;
				sprintf(buf, "#s 0-%d", (int) qty - 1);
				break;

			case AI32SSA_CHAN_ACTIVE_RANGE:

				chans[0]	= last - first + 1;

				if (first == last)
					sprintf(buf, "# %ld", (long) first);
				else
					sprintf(buf, "#s %ld-%ld", (long) first, (long) last);

				break;
		}

		if (errs == 0)
		{
			printf(	"%ld Channel%s  (%s)\n",
					(long) chans[0],
					(chans[0] == 1) ? "" : "s",
					buf);
		}
	}

	return(errs);
}



//*****************************************************************************
static int _read_data(const args_t* args)
{
	int		errs	= 0;
	long	get		= sizeof(_buffer);
	int		got;

	gsc_label("Reading");
	got	= ai32ssa_read(args->fd, _buffer, get);

	if (got < 0)
	{
		errs	= 1;
	}
	else if (got != get)
	{
		errs	= 1;
		printf(	"FAIL <---  (got %ld bytes, requested %ld)\n",
				(long) got,
				(long) get);
	}
	else
	{
		errs	= 0;
		printf(	"PASS  (%ld bytes)\n",
				(long) get);
	}

	return(errs);
}



//*****************************************************************************
static int _save_data(int errs)
{
	FILE*		file;
	int			i;
	long		l;
	const char*	name	= "data.txt";
	long		samples	= sizeof(_buffer) / 4;

	gsc_label("Saving");

	for (;;)
	{
		if (errs)
		{
			printf("SKIPPED  (errors)\n");
			errs	= 0;
			break;
		}

		file	= fopen(name, "w+b");

		if (file == NULL)
		{
			printf("FAIL <---  (unable to create %s)\n", name);
			errs	= 1;
			break;
		}

		for (l = 0; l < samples; l++)
		{
			if ((l) && (_buffer[l] & 0x10000))
			{
				i	= fprintf(file, "\r\n");

				if (i != 2)
				{
					printf("FAIL <---  (fprintf() failure to %s)\n", name);;
					errs	= 1;
					break;
				}
			}

			i	= fprintf(file, "  %08lX", (long) _buffer[l]);

			if (i != 10)
			{
				printf("FAIL <---  (fprintf() failure to %s)\n", name);
				errs	= 1;
				break;
			}
		}

		fclose(file);

		if (errs == 0)
			printf("PASS  (%s)\n", name);

		break;
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	s32	chans	= 32;
	int	errs	= 0;

	gsc_label("Capture & Save");
	printf("\n");
	gsc_label_level_inc();

	errs	+= ai32ssa_config_ain		(args->fd, -1, 1, 4000);
	errs	+= ai32ssa_rx_io_mode		(args->fd, -1, 1, args->io_mode, NULL);
	errs	+= _channels(args, &chans);
	errs	+= ai32ssa_ain_buf_clear	(args->fd, -1, 1);
	errs	+= _read_data(args);
	errs	+= ai32ssa_ain_buf_overflow	(args->fd, -1, 1, -1, NULL);
	errs	+= ai32ssa_ain_buf_underflow(args->fd, -1, 1, -1, NULL);
	errs	+= _save_data(errs);

	gsc_label_level_dec();
	return(errs);
}



