// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AI32SSA/samples/sbtest/ain_buf_level.c $
// $Rev: 51919 $
// $Date: 2022-10-20 17:31:15 -0500 (Thu, 20 Oct 2022) $

// 16AI32SSA: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	// There are no persistent bits to test with this service.
	return(0);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list[]	=
	{
		// Put some data in the FIFO: unpacked, 16 samples, fifo size = 8

		{				// Scan channels 0-7
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_CHAN_ACTIVE,
			/* arg		*/	AI32SSA_CHAN_ACTIVE_0_7,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// ADC source = BCR Control Bit
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_SAMP_CLK_SRC,
			/* arg		*/	AI32SSA_SAMP_CLK_SRC_BCR,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Perform an Input Sync operation.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_INPUT_SYNC,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Wait for data reception.
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	50,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Perform an Input Sync operation.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_INPUT_SYNC,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Wait for data reception.
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	50,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int			errs	= 0;
	int			i;
	s32			lvl;
	gsc_reg_t	reg;
	int			ret;

	errs	+= ai32ssa_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_reg_list(fd, list);

	// We're only testing the level accuracy from 16 samples to zero.

	for (i = 16; (errs == 0) && (i >= 0); i--)
	{
		ret	= ai32ssa_ioctl(fd, AI32SSA_IOCTL_AIN_BUF_LEVEL, &lvl);

		if (ret)
		{
			errs++;
			break;
		}
		else if (lvl != i)
		{
			errs++;
			printf("FAIL <---  (expected %d, got %ld)\n", i, (long) lvl);
			break;
		}

		reg.reg	= AI32SSA_GSC_IDBR;
		ret		= ai32ssa_ioctl(fd, AI32SSA_IOCTL_REG_READ, &reg);

		if (ret)
		{
			errs++;
			break;
		}
	}

	errs	+= ai32ssa_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	ain_buf_level_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AI32SSA_IOCTL_AIN_BUF_LEVEL.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ain_buf_level_test(int fd)
{
	int	errs	= 0;

	gsc_label("AI32SSA_IOCTL_AIN_BUF_LEVEL");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


