// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AI32SSA/samples/sbtest/ain_buf_overflow.c $
// $Rev: 51919 $
// $Date: 2022-10-20 17:31:15 -0500 (Thu, 20 Oct 2022) $

// 16AI32SSA: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	// There are no persistent bits to test with this service.
	return(0);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list_1[]	=
	{
		{				// ADC source = Rate-A Generator.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_SAMP_CLK_SRC,
			/* arg		*/	AI32SSA_SAMP_CLK_SRC_RAG,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Enable Rate-A Generator
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_RAG_ENABLE,
			/* arg		*/	AI32SSA_GEN_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Wait to see if we get any data.
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Disable Rate-A Generator
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_RAG_ENABLE,
			/* arg		*/	AI32SSA_GEN_ENABLE_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Verify that we don't have an overflow.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AI32SSA_GSC_BCTLR,
			/* mask		*/	0x20000,
			/* value	*/	0x00000
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_2[]	=
	{
		{				// Enable Rate-A Generator
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_RAG_ENABLE,
			/* arg		*/	AI32SSA_GEN_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Wait to see if we get any data.
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	50,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Disable Rate-A Generator
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_RAG_ENABLE,
			/* arg		*/	AI32SSA_GEN_ENABLE_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_3[]	=
	{
		{				// Verify that we do have an overflow.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AI32SSA_GSC_BCTLR,
			/* mask		*/	0x20000,
			/* value	*/	0x20000
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_4[]	=
	{
		{				// Verify that CLEAR removes the status.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_AIN_BUF_OVERFLOW,
			/* arg		*/	AI32SSA_AIN_BUF_OVERFLOW_CLEAR,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Verify that CLEAR removes the status.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AI32SSA_GSC_BCTLR,
			/* mask		*/	0x20000,
			/* value	*/	0x00000
		},

		{ SERVICE_END_LIST }
	};

	u32	bcr;
	int	errs	= 0;
	int	i;
	int	ret;
	s32	sts		= 0;

	errs	+= ai32ssa_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_reg_list(fd, list_1);

	if (errs == 0)
	{
		// Verify that the status is NO overflow.
		sts	= AI32SSA_AIN_BUF_OVERFLOW_CHECK;
		ret	= ai32ssa_ioctl(fd, AI32SSA_IOCTL_AIN_BUF_OVERFLOW, &sts);

		if (ret)
		{
			errs++;
		}
		else if (sts != AI32SSA_AIN_BUF_OVERFLOW_NO)
		{
			errs++;
			printf(	"FAIL <---  (%d. overflow occurred but was unexpected)\n",
					__LINE__);
		}
	}

	for (i = 0; (errs == 0) && (i < 100); i++)
	{
		errs	+= service_ioctl_set_reg_list(fd, list_2);
		errs	+= ai32ssa_reg_read(fd, -1, 0, AI32SSA_GSC_BCTLR, &bcr);

		if (bcr & 0x20000)
			break;
	}

	errs	+= service_ioctl_set_reg_list(fd, list_3);

	if (errs == 0)
	{
		// Verify that the status IS an overflow.
		sts	= AI32SSA_AIN_BUF_OVERFLOW_CHECK;
		ret	= ai32ssa_ioctl(fd, AI32SSA_IOCTL_AIN_BUF_OVERFLOW, &sts);

		if (ret)
		{
			errs++;
		}
		else if (sts != AI32SSA_AIN_BUF_OVERFLOW_YES)
		{
			errs++;
			printf(	"FAIL <---  (%d. overflow expected but did not occur)\n",
					__LINE__);
		}
	}

	errs	+= service_ioctl_set_reg_list(fd, list_4);

	if (errs == 0)
	{
		// Verify that the status is NO overflow.
		sts	= AI32SSA_AIN_BUF_OVERFLOW_CHECK;
		ret	= ai32ssa_ioctl(fd, AI32SSA_IOCTL_AIN_BUF_OVERFLOW, &sts);

		if (ret)
		{
			errs++;
		}
		else if (sts != AI32SSA_AIN_BUF_OVERFLOW_NO)
		{
			errs++;
			printf(	"FAIL <---  (%d. overflow occurred but was unexpected)\n",
					__LINE__);
		}
	}

	errs	+= ai32ssa_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	ain_buf_overflow_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AI32SSA_IOCTL_AIN_BUF_OVERFLOW.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ain_buf_overflow_test(int fd)
{
	int	errs	= 0;

	gsc_label("AI32SSA_IOCTL_AIN_BUF_OVERFLOW");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


