// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AI32SSA/samples/sbtest/ain_buf_thr_sts.c $
// $Rev: 51919 $
// $Date: 2022-10-20 17:31:15 -0500 (Thu, 20 Oct 2022) $

// 16AI32SSA: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	// There are no persistent bits to test with this service.
	return(0);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list_1[]	=
	{
		// Put some data in the FIFO: unpacked, 16 samples

		{				// Scan channels 0-7
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_CHAN_ACTIVE,
			/* arg		*/	AI32SSA_CHAN_ACTIVE_0_7,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// ADC source = BCR.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_SAMP_CLK_SRC,
			/* arg		*/	AI32SSA_SAMP_CLK_SRC_BCR,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Perform an Input Sync operation.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_INPUT_SYNC,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Wait for data reception.
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	50,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Perform an Input Sync operation.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_INPUT_SYNC,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Wait for data reception.
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	50,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	service_data_t	list_2[]	=
	{
		{	// 0		// Set the threshold to the current fill level + 1.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_AIN_BUF_THR_LVL,
			/* arg		*/	0,	// SET THIS FOR EACH LOOP ITERATION.
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 1		// Verify that the threshold flag is clear.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AI32SSA_GSC_IBCR,
			/* mask		*/	0x80000,
			/* value	*/	0x00000
		},
		{	// 2		// Set the threshold to the current fill level + 0.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_AIN_BUF_THR_LVL,
			/* arg		*/	0,	// SET THIS FOR EACH LOOP ITERATION.
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 3		// Verify that the threshold flag is clear.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AI32SSA_GSC_IBCR,
			/* mask		*/	0x80000,
			/* value	*/	0x00000
		},
		{	// 4		// Set the threshold to the current fill level - 1.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_AIN_BUF_THR_LVL,
			/* arg		*/	0,	// SET THIS FOR EACH LOOP ITERATION.
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 5		// Verify that the threshold flag is set.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AI32SSA_GSC_IBCR,
			/* mask		*/	0x80000,
			/* value	*/	0x80000
		},

		{ SERVICE_END_LIST }
	};

	service_data_t	list_3[]	=
	{
		{				// Read a value from the input buffer.
			/* service	*/	SERVICE_REG_READ,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AI32SSA_GSC_IDBR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Verify that the threshold flag is clear.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AI32SSA_GSC_IBCR,
			/* mask		*/	0x80000,
			/* value	*/	0x00000
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	i;
	int	ret;
	s32	v;

	errs	+= ai32ssa_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_reg_list(fd, list_1);

	// We're only testing the status accuracy for a small range.

	for (i = 16; (errs == 0) && (i >= 3); i--)
	{
		list_2[0].arg	= i + 1;
		list_2[2].arg	= i;
		list_2[4].arg	= i - 1;

		// Verify that the status is SET.
		errs	+= service_ioctl_set_reg_list(fd, list_2);
		ret		= ai32ssa_ioctl(fd, AI32SSA_IOCTL_AIN_BUF_THR_STS, &v);

		if (ret)
		{
			errs++;
		}
		else if (v != AI32SSA_AIN_BUF_THR_STS_SET)
		{
			errs++;
			printf(	"FAIL <---  (%d. status is CLEAR but should be SET)\n",
					__LINE__);
		}

		// Verify that the status is CLEAR.
		errs	+= service_ioctl_set_reg_list(fd, list_3);
		ret		= ai32ssa_ioctl(fd, AI32SSA_IOCTL_AIN_BUF_THR_STS, &v);

		if (ret)
		{
			errs++;
		}
		else if (v != AI32SSA_AIN_BUF_THR_STS_CLEAR)
		{
			errs++;
			printf(	"FAIL <---  (%d. status is SET but should be CLEAR)\n",
					__LINE__);
		}
	}

	errs	+= ai32ssa_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	ain_buf_thr_sts_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AI32SSA_IOCTL_AIN_BUF_THR_STS.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ain_buf_thr_sts_test(int fd)
{
	int	errs	= 0;

	gsc_label("AI32SSA_IOCTL_AIN_BUF_THR_STS");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


