// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AI32SSA/samples/sbtest/ain_buf_underflow.c $
// $Rev: 51919 $
// $Date: 2022-10-20 17:31:15 -0500 (Thu, 20 Oct 2022) $

// 16AI32SSA: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	// There are no persistent bits to test with this service.
	return(0);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list_1[]	=
	{
		{				// Verify that we don't have an underflow.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AI32SSA_GSC_BCTLR,
			/* mask		*/	0x10000,
			/* value	*/	0x00000
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_2[]	=
	{
		{				// Force an under flow.
			/* service	*/	SERVICE_REG_READ,
			/* cmd		*/	-1,
			/* arg		*/	-1,
			/* reg		*/	AI32SSA_GSC_IDBR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Verify that we do have an under flow.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	-1,
			/* arg		*/	-1,
			/* reg		*/	AI32SSA_GSC_BCTLR,
			/* mask		*/	0x10000,
			/* value	*/	0x10000
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_3[]	=
	{
		{				// Verify that CLEAR removes the status.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_AIN_BUF_UNDERFLOW,
			/* arg		*/	AI32SSA_AIN_BUF_UNDERFLOW_CLEAR,
			/* reg		*/	AI32SSA_GSC_BCTLR,
			/* mask		*/	0x10000,
			/* value	*/	0x00000
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	ret;
	s32	sts;

	errs	+= ai32ssa_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_reg_list(fd, list_1);

	if (errs == 0)
	{
		// Verify that the status is NO underflow.
		sts	= AI32SSA_AIN_BUF_UNDERFLOW_CHECK;
		ret	= ai32ssa_ioctl(fd, AI32SSA_IOCTL_AIN_BUF_UNDERFLOW, &sts);

		if (ret)
		{
			errs++;
		}
		else if (sts != AI32SSA_AIN_BUF_UNDERFLOW_NO)
		{
			errs++;
			printf(	"FAIL <---  (%d. underflow occurred but unexpected)\n",
					__LINE__);
		}
	}

	errs	+= service_ioctl_set_reg_list(fd, list_2);

	if (errs == 0)
	{
		// Verify that the status IS an underflow.
		sts	= AI32SSA_AIN_BUF_UNDERFLOW_CHECK;
		ret	= ai32ssa_ioctl(fd, AI32SSA_IOCTL_AIN_BUF_UNDERFLOW, &sts);

		if (ret)
		{
			errs++;
		}
		else if (sts != AI32SSA_AIN_BUF_UNDERFLOW_YES)
		{
			errs++;
			printf(	"FAIL <---  (%d. underflow expected but did not occur)\n",
					__LINE__);
		}
	}

	errs	+= service_ioctl_set_reg_list(fd, list_3);

	if (errs == 0)
	{
		// Verify that the status is NO underflow.
		sts	= AI32SSA_AIN_BUF_UNDERFLOW_CHECK;
		ret	= ai32ssa_ioctl(fd, AI32SSA_IOCTL_AIN_BUF_UNDERFLOW, &sts);

		if (ret)
		{
			errs++;
		}
		else if (sts != AI32SSA_AIN_BUF_UNDERFLOW_NO)
		{
			errs++;
			printf(	"FAIL <---  (%d. underflow occurred but unexpected)\n",
					__LINE__);
		}
	}

	errs	+= ai32ssa_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	ain_buf_underflow_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AI32SSA_IOCTL_AIN_BUF_UNDERFLOW.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ain_buf_underflow_test(int fd)
{
	int	errs	= 0;

	gsc_label("AI32SSA_IOCTL_AIN_BUF_UNDERFLOW");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


