// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AI32SSA/samples/sbtest/burst_size.c $
// $Rev: 51919 $
// $Date: 2022-10-20 17:31:15 -0500 (Thu, 20 Oct 2022) $

// 16AI32SSA: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_BURST_SIZE,
			/* arg		*/	-1,
			/* reg		*/	AI32SSA_GSC_BSIZR,
			/* mask		*/	0xFFFFF,
			/* value	*/	-1
		},

		{ SERVICE_END_LIST }
	};

	int errs	= 0;
	int	i;

	errs	+= ai32ssa_initialize(fd, -1, 0);

	for (i = 0; i < 32; i++)
	{
		// Walking one.
		list[0].arg		= (0x1L << i) & 0xFFFFF;
		list[0].value	= list[0].arg;

		errs	+= service_ioctl_set_reg_list(fd, list);
		errs	+= service_ioctl_reg_get_list(fd, list);

		errs	+= service_ioctl_set_reg_list(fd, list);
		errs	+= service_ioctl_reg_get_list(fd, list);

		// Walking zero.
		list[0].arg		= (0xFFFFF ^ (0x1L << i)) & 0xFFFFF;
		list[0].value	= list[0].arg;

		errs	+= service_ioctl_set_reg_list(fd, list);
		errs	+= service_ioctl_reg_get_list(fd, list);

		errs	+= service_ioctl_set_reg_list(fd, list);
		errs	+= service_ioctl_reg_get_list(fd, list);
	}

	errs	+= ai32ssa_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	service_data_t	list[]	=
	{
		{	// 0
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_BURST_SIZE,
			/* arg		*/	-1,	// specified below
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 1		// Scan channels 0-7
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_CHAN_ACTIVE,
			/* arg		*/	AI32SSA_CHAN_ACTIVE_0_7,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 2
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_BURST_SRC,
			/* arg		*/	AI32SSA_BURST_SRC_BCR,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 3
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_SAMP_CLK_SRC,
			/* arg		*/	AI32SSA_SAMP_CLK_SRC_RAG,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 4
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_RAG_ENABLE,
			/* arg		*/	AI32SSA_GEN_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 5		// Perform an Input Sync operation.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_INPUT_SYNC,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 6		// Wait for operation activity.
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	50,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 7		// Verify that we got the expected data.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AI32SSA_GSC_BUFSR,
			/* mask		*/	0x7FFFF,
			/* value	*/	0	// updated below
		},

		{ SERVICE_END_LIST }
	};

	int		errs	= 0;
	int		i;

	errs	+= ai32ssa_initialize(fd, -1, 0);

	for (i = 1; i < 32; i++)
	{
		list[0].arg		= i;
		list[7].value	+= i * 8;
		errs			+= service_ioctl_set_reg_list(fd, list);
	}

	errs	+= ai32ssa_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	burst_size_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AI32SSA_IOCTL_BURST_SIZE.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int burst_size_test(int fd)
{
	int	errs	= 0;

	gsc_label("AI32SSA_IOCTL_BURST_SIZE");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


