// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AI32SSA/samples/sbtest/dmdma_mode.c $
// $Rev: 51919 $
// $Date: 2022-10-20 17:31:15 -0500 (Thu, 20 Oct 2022) $

// 16AI32SSA: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_DMDMA_MODE,
			/* arg		*/	AI32SSA_DMDMA_MODE_THRESHOLD,
			/* reg		*/	AI32SSA_GSC_BCTLR,
			/* mask		*/	0x80000,
			/* value	*/	0x00000
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_DMDMA_MODE,
			/* arg		*/	AI32SSA_DMDMA_MODE_AUTONOMOUS,
			/* reg		*/	AI32SSA_GSC_BCTLR,
			/* mask		*/	0x80000,
			/* value	*/	0x80000
		},

		{ SERVICE_END_LIST }
	};

	int errs	= 0;

	errs	+= ai32ssa_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_reg_list(fd, list);
	errs	+= service_ioctl_reg_get_list(fd, list);

	errs	+= service_ioctl_set_reg_list(fd, list);
	errs	+= service_ioctl_reg_get_list(fd, list);

	errs	+= ai32ssa_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _function_sub_test(int fd, s32 mode, int qty)
{
	const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_BURST_SIZE,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Scan channels 0-7
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_CHAN_ACTIVE,
			/* arg		*/	AI32SSA_CHAN_ACTIVE_0_7,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_BURST_SRC,
			/* arg		*/	AI32SSA_BURST_SRC_BCR,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_SAMP_CLK_SRC,
			/* arg		*/	AI32SSA_SAMP_CLK_SRC_RAG,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_RAG_NRATE,
			/* arg		*/	200,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_RAG_ENABLE,
			/* arg		*/	AI32SSA_GEN_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Perform an Input Sync operation.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_INPUT_SYNC,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Wait for operation activity.
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	50,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Verify that we got the expected data.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AI32SSA_GSC_BUFSR,
			/* mask		*/	0x7FFFF,
			/* value	*/	800
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_DMDMA_MODE,
			/* arg		*/	mode,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_RX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_DMDMA,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_AIN_BUF_THR_LVL,
			/* arg		*/	900,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_RX_IO_TIMEOUT,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	char	buf[1024];
	int		errs	= 0;
	int		ret;

	errs	+= ai32ssa_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_reg_list(fd, list);

	if (errs == 0)
	{
		ret	= ai32ssa_read(fd, buf, 800);

		if (ret == qty)
		{
			// This is as expected.
		}
		else
		{
			errs++;
			printf(	"FAIL <---  (%d. ai32ssa_read() %ld bytes, expected %ld)\n",
					__LINE__,
					(long) ret,
					(long) qty);
		}
	}

	errs	+= ai32ssa_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	int	errs	= 0;

	errs	+= _function_sub_test(fd, AI32SSA_DMDMA_MODE_THRESHOLD, -110);
	errs	+= _function_sub_test(fd, AI32SSA_DMDMA_MODE_AUTONOMOUS, 800);

	return(errs);
}



/******************************************************************************
*
*	Function:	dmdma_mode_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AI32SSA_IOCTL_DMDMA_MODE.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int dmdma_mode_test(int fd)
{
	int	errs	= 0;

	gsc_label("AI32SSA_IOCTL_DMDMA_MODE");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


