// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AI32SSA/samples/sbtest/rbg_clk_src.c $
// $Rev: 51919 $
// $Date: 2022-10-20 17:31:15 -0500 (Thu, 20 Oct 2022) $

// 16AI32SSA: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_RBG_CLK_SRC,
			/* arg		*/	AI32SSA_RBG_CLK_SRC_MASTER,
			/* reg		*/	AI32SSA_GSC_SSCR,
			/* mask		*/	0x400,
			/* value	*/	0x000
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_RBG_CLK_SRC,
			/* arg		*/	AI32SSA_RBG_CLK_SRC_RAG,
			/* reg		*/	AI32SSA_GSC_SSCR,
			/* mask		*/	0x400,
			/* value	*/	0x400
		},

		{ SERVICE_END_LIST }
	};

	int errs	= 0;

	errs	+= ai32ssa_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_reg_list(fd, list);
	errs	+= service_ioctl_reg_get_list(fd, list);

	errs	+= service_ioctl_set_reg_list(fd, list);
	errs	+= service_ioctl_reg_get_list(fd, list);

	errs	+= ai32ssa_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_RBG_ENABLE,
			/* arg		*/	AI32SSA_GEN_ENABLE_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// ADC source = Rate-B Generator.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_SAMP_CLK_SRC,
			/* arg		*/	AI32SSA_SAMP_CLK_SRC_RBG,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Set the threshold very low.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_AIN_BUF_THR_LVL,
			/* arg		*/	3,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		// MASTER CLOCK

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_RBG_ENABLE,
			/* arg		*/	AI32SSA_GEN_ENABLE_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Clear the buffer.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_AIN_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Disable Rate-A Generator
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_RAG_ENABLE,
			/* arg		*/	AI32SSA_GEN_ENABLE_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Rate-B Source: Master Clock
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_RBG_CLK_SRC,
			/* arg		*/	AI32SSA_RBG_CLK_SRC_MASTER,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_RBG_ENABLE,
			/* arg		*/	AI32SSA_GEN_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Wait a moment to see if we get any data.
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	50,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Verify that we got data.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AI32SSA_GSC_IBCR,
			/* mask		*/	0x80000,
			/* value	*/	0x80000
		},

		// RATE-A GENERATOR

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_RBG_ENABLE,
			/* arg		*/	AI32SSA_GEN_ENABLE_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Clear the buffer.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_AIN_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Disable Rate-A Generator
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_RAG_ENABLE,
			/* arg		*/	AI32SSA_GEN_ENABLE_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Rate-B Source: Rate-A Generator
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_RBG_CLK_SRC,
			/* arg		*/	AI32SSA_RBG_CLK_SRC_RAG,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_RBG_ENABLE,
			/* arg		*/	AI32SSA_GEN_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Wait a moment to see if we get any data.
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	50,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Verify that we got no data.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AI32SSA_GSC_IBCR,
			/* mask		*/	0x80000,
			/* value	*/	0x00000
		},

		{				// Enable Rate-A Generator
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AI32SSA_IOCTL_RAG_ENABLE,
			/* arg		*/	AI32SSA_GEN_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Wait a moment to see if we get any data.
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	50,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Verify that we got no data.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	AI32SSA_GSC_IBCR,
			/* mask		*/	0x80000,
			/* value	*/	0x80000
		},

		{ SERVICE_END_LIST }
	};

	int errs	= 0;

	errs	+= ai32ssa_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_reg_list(fd, list);
	errs	+= ai32ssa_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	rbg_clk_src_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AI32SSA_IOCTL_RBG_CLK_SRC.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int rbg_clk_src_test(int fd)
{
	int	errs	= 0;

	gsc_label("AI32SSA_IOCTL_RBG_CLK_SRC");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


