// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AI32SSA/utils/util_config_ain.c $
// $Rev: 51925 $
// $Date: 2022-10-20 17:37:49 -0500 (Thu, 20 Oct 2022) $

// 16AI32SSA: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ai32ssa_config_ain
*
*	Purpose:
*
*		Configure the Analog Input services.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		nrate	The desired Nrate value.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int	ai32ssa_config_ain(int fd, int index, int verbose, s32 nrate)
{
	int	errs	= 0;
	s32	hv		= 0;
	s32	qty		= 32;
	s32	range;

	if (verbose)
	{
		gsc_label_index("Input Configuration", index);
		printf("\n");
		gsc_label_level_inc();
	}

	errs	+= ai32ssa_query				(fd, index, verbose, AI32SSA_QUERY_V_RANGE_HIGH, &hv);
	errs	+= ai32ssa_query				(fd, index, verbose, AI32SSA_QUERY_CHANNEL_QTY, &qty);
	range	= hv ? AI32SSA_AIN_RANGE_10V : AI32SSA_AIN_RANGE_2_5V;

	errs	+= ai32ssa_initialize			(fd, index, verbose);
	errs	+= ai32ssa_rx_io_mode			(fd, index, verbose, GSC_IO_MODE_PIO,				NULL);
	errs	+= ai32ssa_rx_io_overflow		(fd, index, verbose, AI32SSA_IO_OVERFLOW_CHECK,		NULL);
	errs	+= ai32ssa_rx_io_timeout		(fd, index, verbose, AI32SSA_IO_TIMEOUT_DEFAULT,	NULL);
	errs	+= ai32ssa_rx_io_underflow		(fd, index, verbose, AI32SSA_IO_UNDERFLOW_CHECK,	NULL);

	errs	+= ai32ssa_samp_clk_src			(fd, index, verbose, AI32SSA_SAMP_CLK_SRC_RAG,		NULL);
	errs	+= ai32ssa_ain_buf_thr_lvl		(fd, index, verbose, 250000,						NULL);
	errs	+= ai32ssa_ain_mode				(fd, index, verbose, AI32SSA_AIN_MODE_SYSTEM,		NULL);
	errs	+= ai32ssa_ain_range			(fd, index, verbose, range,							NULL);
	errs	+= ai32ssa_burst_size			(fd, index, verbose, 1,								NULL);
	errs	+= ai32ssa_burst_src			(fd, index, verbose, AI32SSA_BURST_SRC_DISABLE,		NULL);
	errs	+= ai32ssa_chan_active			(fd, index, verbose, AI32SSA_CHAN_ACTIVE_RANGE,		NULL);
	errs	+= ai32ssa_chan_first			(fd, index, verbose, 0,								NULL);
	errs	+= ai32ssa_chan_last			(fd, index, verbose, qty - 1,						NULL);
	errs	+= ai32ssa_chan_single			(fd, index, verbose, 0,								NULL);
	errs	+= ai32ssa_data_format			(fd, index, verbose, AI32SSA_DATA_FORMAT_OFF_BIN,	NULL);
	errs	+= ai32ssa_data_packing			(fd, index, verbose, AI32SSA_DATA_PACKING_DISABLE,	NULL);
	errs	+= ai32ssa_dmdma_mode			(fd, index, verbose, AI32SSA_DMDMA_MODE_AUTONOMOUS,	NULL);
	errs	+= ai32ssa_irq0_sel				(fd, index, verbose, AI32SSA_IRQ0_INIT_DONE,		NULL);
	errs	+= ai32ssa_irq1_sel				(fd, index, verbose, AI32SSA_IRQ1_NONE,				NULL);
	errs	+= ai32ssa_rag_nrate			(fd, index, verbose, nrate,							NULL);
	errs	+= ai32ssa_rag_enable			(fd, index, verbose, AI32SSA_GEN_ENABLE_YES,		NULL);
	errs	+= ai32ssa_rbg_nrate			(fd, index, verbose, 500,							NULL);
	errs	+= ai32ssa_rbg_clk_src			(fd, index, verbose, AI32SSA_RBG_CLK_SRC_MASTER,	NULL);
	errs	+= ai32ssa_rbg_enable			(fd, index, verbose, AI32SSA_GEN_ENABLE_NO,			NULL);
	errs	+= ai32ssa_scan_marker			(fd, index, verbose, AI32SSA_SCAN_MARKER_DISABLE,	NULL);
	errs	+= ai32ssa_scan_marker_val		(fd, index, verbose, 0,								NULL);
	errs	+= ai32ssa_fsamp_ain_report_all	(fd, index, verbose,								NULL);
//	errs	+= ai32ssa_auto_calibrate		(fd, index, verbose);	// Cable may need disconnecting.
	errs	+= ai32ssa_ain_buf_clear		(fd, index, verbose);

	if (verbose)
		gsc_label_level_dec();

	return(errs);
}


