// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AI32SSA/utils/util_fsamp_ain.c $
// $Rev: 51925 $
// $Date: 2022-10-20 17:37:49 -0500 (Thu, 20 Oct 2022) $

// 16AI32SSA: Utilities: source file

#include "main.h"



//*****************************************************************************
static int _channels(int fd, s32* first, s32* last)
{
	s32	active	= -1;
	int	errs	= 0;
	s32	qty;
	s32	single	= -1;

	first[0]	= -1;
	last[0]		= -1;

	errs	+= ai32ssa_query(fd, -1, 0, AI32SSA_QUERY_CHANNEL_QTY,		&qty);
	errs	+= ai32ssa_ioctl_util(fd, -1, 0, AI32SSA_IOCTL_CHAN_ACTIVE,	&active	);
	errs	+= ai32ssa_ioctl_util(fd, -1, 0, AI32SSA_IOCTL_CHAN_FIRST,	first	);
	errs	+= ai32ssa_ioctl_util(fd, -1, 0, AI32SSA_IOCTL_CHAN_LAST,	last	);
	errs	+= ai32ssa_ioctl_util(fd, -1, 0, AI32SSA_IOCTL_CHAN_SINGLE,	&single	);

	switch (active)
	{
		default:

			if (errs == 0)
				printf("FAIL <---  (invalid ACTIVE selection: %ld)\n", (long) active);

			errs++;
			break;

		case AI32SSA_CHAN_ACTIVE_0_1:		first[0]	= 0;		last[0]	= 1;		break;
		case AI32SSA_CHAN_ACTIVE_0_3:		first[0]	= 0;		last[0]	= 3;		break;
		case AI32SSA_CHAN_ACTIVE_0_7:		first[0]	= 0;		last[0]	= 7;		break;
		case AI32SSA_CHAN_ACTIVE_0_15:		first[0]	= 0;		last[0]	= 15;		break;
		case AI32SSA_CHAN_ACTIVE_0_3X:		first[0]	= 0;		last[0]	= qty - 1;	break;
		case AI32SSA_CHAN_ACTIVE_SINGLE:	first[0]	= single;	last[0]	= single;	break;
		case AI32SSA_CHAN_ACTIVE_RANGE:													break;
	}

	return(errs);
}



//*****************************************************************************
static int _rag_disabled(s32* sps, char* buf)
{
	int	errs	= 0;
	s32	fsamp	= 0;

	strcpy(buf, "Rate-A Generator Disabled");

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



//*****************************************************************************
static int _rbg_disabled(s32* sps, char* buf)
{
	int	errs	= 0;
	s32	fsamp	= 0;

	strcpy(buf, "Rate-B Generator Disabled");

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



//*****************************************************************************
static int _clk_rag_mc(int fd, s32* sps, char* buf)
{
	s32	a_enable	= -1;
	s32	a_nrate		= -1;
	int	errs		= 0;
	s32	fsamp		= 0;
	s32	max;
	s32	ref;

	errs	+= ai32ssa_query(fd, -1, 0, AI32SSA_QUERY_MASTER_CLOCK,	&ref);
	errs	+= ai32ssa_query(fd, -1, 0, AI32SSA_QUERY_FSAMP_MAX,		&max);
	errs	+= ai32ssa_ioctl_util(fd, -1, 0, AI32SSA_IOCTL_RAG_ENABLE,	&a_enable);
	errs	+= ai32ssa_ioctl_util(fd, -1, 0, AI32SSA_IOCTL_RAG_NRATE,	&a_nrate);

	for (;;)	// A convenience loop.
	{
		if (a_enable == AI32SSA_GEN_ENABLE_NO)
		{
			errs	+= _rag_disabled(&fsamp, buf);
			break;
		}

		sprintf(buf, "Fref %ld, A Nrate %ld", (long) ref, (long) a_nrate);

		// Rate-A Generator calculations
		fsamp	= ref / a_nrate;

		if (fsamp > max)
			strcat(buf, "  <--- UNSTABLE");

		break;
	}

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



//*****************************************************************************
static int _clk_rbg_mc(int fd, s32* sps, char* buf)
{
	s32	b_enable	= -1;
	s32	b_nrate		= -1;
	int	errs		= 0;
	s32	fsamp		= 0;
	s32	max;
	s32	ref;

	errs	+= ai32ssa_query(fd, -1, 0, AI32SSA_QUERY_MASTER_CLOCK,	&ref);
	errs	+= ai32ssa_query(fd, -1, 0, AI32SSA_QUERY_FSAMP_MAX,		&max);
	errs	+= ai32ssa_ioctl_util(fd, -1, 0, AI32SSA_IOCTL_RBG_ENABLE,	&b_enable);
	errs	+= ai32ssa_ioctl_util(fd, -1, 0, AI32SSA_IOCTL_RBG_NRATE,	&b_nrate);

	for (;;)	// A convenience loop.
	{
		if (b_enable == AI32SSA_GEN_ENABLE_NO)
		{
			errs	+= _rbg_disabled(&fsamp, buf);
			break;
		}

		sprintf(buf, "Fref %ld, B Nrate %ld", (long) ref, (long) b_nrate);

		// Rate-A Generator calculations
		fsamp	= ref / b_nrate;

		if (fsamp > max)
			strcat(buf, "  <--- UNSTABLE");

		break;
	}

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



//*****************************************************************************
static int _clk_rbg_rag_mc(int fd, s32* sps, char* buf)
{
	s32	a_enable	= -1;
	s32	a_nrate		= -1;
	s32	b_enable	= -1;
	s32	b_nrate		= -1;
	int	errs		= 0;
	s32	fsamp		= 0;
	s32	max;
	s32	ref;

	errs	+= ai32ssa_query(fd, -1, 0, AI32SSA_QUERY_MASTER_CLOCK,	&ref);
	errs	+= ai32ssa_query(fd, -1, 0, AI32SSA_QUERY_FSAMP_MAX,		&max);
	errs	+= ai32ssa_ioctl_util(fd, -1, 0, AI32SSA_IOCTL_RAG_ENABLE,	&a_enable);
	errs	+= ai32ssa_ioctl_util(fd, -1, 0, AI32SSA_IOCTL_RAG_NRATE,	&a_nrate);
	errs	+= ai32ssa_ioctl_util(fd, -1, 0, AI32SSA_IOCTL_RBG_ENABLE,	&b_enable);
	errs	+= ai32ssa_ioctl_util(fd, -1, 0, AI32SSA_IOCTL_RBG_NRATE,	&b_nrate);

	for (;;)	// A convenience loop.
	{
		if (a_enable == AI32SSA_GEN_ENABLE_NO)
		{
			errs	+= _rag_disabled(&fsamp, buf);
			break;
		}

		if (b_enable == AI32SSA_GEN_ENABLE_NO)
		{
			errs	+= _rbg_disabled(&fsamp, buf);
			break;
		}

		sprintf(buf,
				"Fref %ld, A Nrate %ld, B Nrate %ld",
				(long) ref,
				(long) a_nrate,
				(long) b_nrate);

		// Rate-A/B Generator calculations
		fsamp	= ref / (a_nrate * b_nrate);

		if (fsamp > max)
			strcat(buf, "  <--- UNSTABLE");

		break;
	}

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



//*****************************************************************************
static int _clk_rbg(int fd, s32* sps, char* buf)
{
	int	errs	= 0;
	s32	fsamp	= 0;
	s32	rbg		= -1;

	errs	+= ai32ssa_ioctl_util(fd, -1, 0, AI32SSA_IOCTL_RBG_CLK_SRC, &rbg);

	if (errs == 0)
	{
		switch (rbg)
		{
			default:

				sprintf(buf, "%d. INTERNAL ERROR: %ld", __LINE__, (long) rbg);
				break;

			case AI32SSA_RBG_CLK_SRC_MASTER:

				errs	+= _clk_rbg_mc(fd, &fsamp, buf);
				break;

			case AI32SSA_RBG_CLK_SRC_RAG:

				errs	+= _clk_rbg_rag_mc(fd, &fsamp, buf);
				break;
		}
	}

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



//*****************************************************************************
static int _clk_ext(s32* sps, char* buf)
{
	int	errs	= 0;
	s32	fsamp	= 0;

	strcpy(buf, "Externally Clocked Sampling");

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



//*****************************************************************************
static int _clk_bcr(s32* sps, char* buf)
{
	int	errs	= 0;
	s32	fsamp	= 0;

	strcpy(buf, "Software Clocked Sampling");

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



//*****************************************************************************
static int _burst_disable(int fd, s32* sps, char* buf)
{
	int	errs	= 0;
	s32	fsamp	= 0;
	s32	src		= -1;

	errs	+= ai32ssa_ioctl_util(fd, -1, 0, AI32SSA_IOCTL_SAMP_CLK_SRC, &src);

	if (errs == 0)
	{
		switch (src)
		{
			default:

				sprintf(buf, "%d. INTERNAL ERROR: %ld", __LINE__, (long) src);
				break;

			case AI32SSA_SAMP_CLK_SRC_RAG:

				errs	+= _clk_rag_mc(fd, &fsamp, buf);
				break;

			case AI32SSA_SAMP_CLK_SRC_RBG:

				errs	+= _clk_rbg(fd, &fsamp, buf);
				break;

			case AI32SSA_SAMP_CLK_SRC_EXT:

				errs	+= _clk_ext(&fsamp, buf);
				break;

			case AI32SSA_SAMP_CLK_SRC_BCR:

				errs	+= _clk_bcr(&fsamp, buf);
				break;
		}
	}

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



//*****************************************************************************
static int _burst_rbg_mc_clk_rag_mc(int fd, s32* sps, char* buf)
{
	s32			a_enable	= -1;
	s32			a_nrate		= -1;
	s32			b_enable	= -1;
	s32			b_nrate		= -1;
	float		b_tb;				// Burst: Time Burst
	long long	b_tb_ns;			// Burst: Time Burst Nano Seconds.
	long long	b_tt_ns;			// Burst: Time Total Nano Seconds.
	float		bps;				// Bursts Per Second
	int			errs		= 0;
	s32			fsamp		= 0;
	s32			max;
	long		qty;
	s32			ref;
	long long	s_tb_ns;			// Sample: Time Burst Nano Seconds.
	float		s_ts;				// Sample: Time Scan
	long long	s_ts_ns;			// Sample: Time Scan Nano Seconds.
	s32			size		= -1;

	errs	+= ai32ssa_query(fd, -1, 0, AI32SSA_QUERY_MASTER_CLOCK,	&ref);
	errs	+= ai32ssa_query(fd, -1, 0, AI32SSA_QUERY_FSAMP_MAX,		&max);
	errs	+= ai32ssa_ioctl_util(fd, -1, 0, AI32SSA_IOCTL_BURST_SIZE,	&size);
	errs	+= ai32ssa_ioctl_util(fd, -1, 0, AI32SSA_IOCTL_RAG_ENABLE,	&a_enable);
	errs	+= ai32ssa_ioctl_util(fd, -1, 0, AI32SSA_IOCTL_RAG_NRATE,	&a_nrate);
	errs	+= ai32ssa_ioctl_util(fd, -1, 0, AI32SSA_IOCTL_RBG_ENABLE,	&b_enable);
	errs	+= ai32ssa_ioctl_util(fd, -1, 0, AI32SSA_IOCTL_RBG_NRATE,	&b_nrate);

	for (;;)	// A convenience loop.
	{
		if (a_enable == AI32SSA_GEN_ENABLE_NO)
		{
			fsamp	= 0;
			sprintf(buf, "Rate-A Generator disabled");
			break;
		}

		if (b_enable == AI32SSA_GEN_ENABLE_NO)
		{
			fsamp	= 0;
			sprintf(buf, "Rate-B Generator disabled");
			break;
		}

		sprintf(buf,
				"Fref %ld, A Nrate %ld, B Nrate %ld, Burst Size %ld",
				(long) ref,
				(long) a_nrate,
				(long) b_nrate,
				(long) size);

		if (size == 0)
		{
			fsamp	= ref / a_nrate;
			break;
		}

		// Sample clocking
		s_ts	= (float) a_nrate / ref;
		s_ts_ns	= (long long) (s_ts * 1000000000L + 0.5);
		s_tb_ns	= s_ts_ns * size;

		// Burst Clocking
		b_tb	= (float) b_nrate / ref;
		b_tb_ns	= (long long) (b_tb * 1000000000L + 0.5);
		qty		= (long) ((s_tb_ns + s_ts_ns - 1) / b_tb_ns);
		qty		= (qty < 1) ? 1 : qty;
		b_tt_ns	= b_tb_ns * qty;

		// Sample rate.
		bps		= 1.0 / (1.0 * b_tt_ns / 1000000000L);
		fsamp	= (s32) (size * bps);
		break;
	}

	if ((errs == 0) && (fsamp > max))
		strcat(buf, "  <--- UNSTABLE");

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



//*****************************************************************************
static int _burst_rbg_mc_clk_rbg_mc(s32* sps, char* buf)
{
	int	errs	= 0;
	s32	fsamp	= 0;

	strcpy(buf, "Invalid Configuration");

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



//*****************************************************************************
static int _burst_rbg_mc(int fd, s32* sps, char* buf)
{
	s32	adc		= -1;
	int	errs	= 0;
	s32	fsamp	= 0;

	errs	+= ai32ssa_ioctl_util(fd, -1, 0, AI32SSA_IOCTL_SAMP_CLK_SRC, &adc);

	if (errs == 0)
	{
		switch (adc)
		{
			default:

				sprintf(buf, "%d. INTERNAL ERROR: %ld", __LINE__, (long) adc);
				break;

			case AI32SSA_SAMP_CLK_SRC_BCR:

				errs	+= _clk_bcr(&fsamp, buf);
				break;

			case AI32SSA_SAMP_CLK_SRC_EXT:

				errs	+= _clk_ext(&fsamp, buf);
				break;

			case AI32SSA_SAMP_CLK_SRC_RAG:

				errs	+= _burst_rbg_mc_clk_rag_mc(fd, &fsamp, buf);
				break;

			case AI32SSA_SAMP_CLK_SRC_RBG:

				errs	+= _burst_rbg_mc_clk_rbg_mc(&fsamp, buf);
				break;
		}
	}

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



//*****************************************************************************
static int _burst_rbg_rag_mc_clk_rag_mc(int fd, s32* sps, char* buf)
{
	s32			a_enable	= -1;
	s32			a_nrate		= -1;
	s32			b_enable	= -1;
	s32			b_nrate		= -1;
	float		b_tb;				// Burst: Time Burst
	long long	b_tb_ns;			// Burst: Time Burst Nano Seconds.
	long long	b_tt_ns;			// Burst: Time Total Nano Seconds.
	float		bps;				// Bursts Per Second
	int			errs		= 0;
	s32			fsamp		= 0;
	s32			max;
	long		qty;
	s32			ref;
	long long	s_tb_ns;			// Sample: Time Burst Nano Seconds.
	float		s_ts;				// Sample: Time Scan
	long long	s_ts_ns;			// Sample: Time Scan Nano Seconds.
	s32			size		= -1;

	errs	+= ai32ssa_query(fd, -1, 0, AI32SSA_QUERY_MASTER_CLOCK,	&ref);
	errs	+= ai32ssa_query(fd, -1, 0, AI32SSA_QUERY_FSAMP_MAX,		&max);
	errs	+= ai32ssa_ioctl_util(fd, -1, 0, AI32SSA_IOCTL_BURST_SIZE,	&size);
	errs	+= ai32ssa_ioctl_util(fd, -1, 0, AI32SSA_IOCTL_RAG_ENABLE,	&a_enable);
	errs	+= ai32ssa_ioctl_util(fd, -1, 0, AI32SSA_IOCTL_RAG_NRATE,	&a_nrate);
	errs	+= ai32ssa_ioctl_util(fd, -1, 0, AI32SSA_IOCTL_RBG_ENABLE,	&b_enable);
	errs	+= ai32ssa_ioctl_util(fd, -1, 0, AI32SSA_IOCTL_RBG_NRATE,	&b_nrate);

	for (;;)	// A convenience loop.
	{
		if (a_enable == AI32SSA_GEN_ENABLE_NO)
		{
			fsamp	= 0;
			sprintf(buf, "Rate-A Generator disabled");
			break;
		}

		if (b_enable == AI32SSA_GEN_ENABLE_NO)
		{
			fsamp	= 0;
			sprintf(buf, "Rate-B Generator disabled");
			break;
		}

		sprintf(buf,
				"Fref %ld, A Nrate %ld, B Nrate %ld, Burst Size %ld",
				(long) ref,
				(long) a_nrate,
				(long) b_nrate,
				(long) size);

		if (size == 0)
		{
			fsamp	= ref / a_nrate;
			break;
		}

		// Sample clocking
		s_ts	= (float) a_nrate / ref;
		s_ts_ns	= (long long) (s_ts * 1000000000L + 0.5);
		s_tb_ns	= s_ts_ns * size;

		// Burst Clocking
		b_tb	= (s32) (((double) a_nrate * b_nrate) / ref);
		b_tb_ns	= (long long) (b_tb * 1000000000L + 0.5);
		qty		= (long) ((s_tb_ns + s_ts_ns - 1) / b_tb_ns);
		qty		= (qty < 1) ? 1 : qty;
		b_tt_ns	= b_tb_ns * qty;

		// Sample rate.
		bps		= 1.0 / (1.0 * b_tt_ns / 1000000000L);
		fsamp	= (s32) (size * bps);
		break;
	}

	if ((errs == 0) && (fsamp > max))
		strcat(buf, "  <--- UNSTABLE");

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



//*****************************************************************************
static int _burst_rbg_rag_mc_clk_rbg_rag_mc(s32* sps, char* buf)
{
	int	errs	= 0;
	s32	fsamp	= 0;

	strcpy(buf, "Invalid Configuration");

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



//*****************************************************************************
static int _burst_rbg_rag_mc(int fd, s32* sps, char* buf)
{
	int	errs	= 0;
	s32	fsamp	= 0;
	s32	src		= -1;

	errs	+= ai32ssa_ioctl_util(fd, -1, 0, AI32SSA_IOCTL_SAMP_CLK_SRC, &src);

	if (errs == 0)
	{
		switch (src)
		{
			default:

				sprintf(buf, "%d. INTERNAL ERROR: %ld", __LINE__, (long) src);
				break;

			case AI32SSA_SAMP_CLK_SRC_BCR:

				errs	+= _clk_bcr(&fsamp, buf);
				break;

			case AI32SSA_SAMP_CLK_SRC_EXT:

				errs	+= _clk_ext(&fsamp, buf);
				break;

			case AI32SSA_SAMP_CLK_SRC_RAG:

				errs	+= _burst_rbg_rag_mc_clk_rag_mc(fd, &fsamp, buf);
				break;

			case AI32SSA_SAMP_CLK_SRC_RBG:

				errs	+= _burst_rbg_rag_mc_clk_rbg_rag_mc(&fsamp, buf);
				break;
		}
	}

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



//*****************************************************************************
static int _burst_rbg(int fd, s32* sps, char* buf)
{
	int	errs	= 0;
	s32	fsamp	= 0;
	s32	rbg		= -1;

	errs	+= ai32ssa_ioctl_util(fd, -1, 0, AI32SSA_IOCTL_RBG_CLK_SRC, &rbg);

	if (errs == 0)
	{
		switch (rbg)
		{
			default:

				sprintf(buf, "%d. INTERNAL ERROR: %ld", __LINE__, (long) rbg);
				break;

			case AI32SSA_RBG_CLK_SRC_MASTER:

				errs	+= _burst_rbg_mc(fd, &fsamp, buf);
				break;

			case AI32SSA_RBG_CLK_SRC_RAG:

				errs	+= _burst_rbg_rag_mc(fd, &fsamp, buf);
				break;
		}
	}

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



//*****************************************************************************
static int _burst_ext(s32* sps, char* buf)
{
	int	errs	= 0;
	s32	fsamp	= 0;

	strcpy(buf, "Externally Triggered Bursting");

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



//*****************************************************************************
static int _burst_bcr(s32* sps, char* buf)
{
	int	errs	= 0;
	s32	fsamp	= 0;

	strcpy(buf, "Software Triggered Bursting");

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



/******************************************************************************
*
*	Function:	ai32ssa_fsamp_ain_report
*
*	Purpose:
*
*		Determine and report the sample rate for the specified channel.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		chan	The index of the channel of interest.
*
*		quiet	Operate quietly?
*
*		sps		Store the sample rate here, if non-NULL.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int ai32ssa_fsamp_ain_report(int fd, int index, int chan, int quiet, s32* sps)
{
	char	buf[1024];
	s32		burst		= -1;
	int		errs		= 0;
	s32		first;
	s32		fsamp		= 0;
	s32		last;
	s32		qty;

	if (quiet == 0)
	{
		sprintf(buf, "Channel %d", chan);
		gsc_label_index(buf, index);
	}

	for (;;)
	{
		errs	+= ai32ssa_query(fd, -1, 0, AI32SSA_QUERY_CHANNEL_QTY, &qty);

		if (errs)
			break;

		if ((chan < 0) || (chan > (qty - 1)))
		{
			errs++;
			printf("FAIL <---  (invalid channel index: %d)\n", chan);
			break;
		}

		errs	+= _channels(fd, &first, &last);

		if (errs)
			break;

		if ((chan < first) || (chan > last))
		{
			sps[0]	= 0;

			if (quiet == 0)
				printf("SKIPPED  (disabled)\n");

			break;
		}

		errs	+= ai32ssa_ioctl_util(fd, -1, 0, AI32SSA_IOCTL_BURST_SRC, &burst);

		if (errs)
			break;

		switch (burst)
		{
			default:

				sprintf(buf, "%d. INTERNAL ERROR: %ld", __LINE__, (long) burst);
				break;

			case AI32SSA_BURST_SRC_DISABLE:

				errs	+= _burst_disable(fd, &fsamp, buf);
				break;

			case AI32SSA_BURST_SRC_RBG:

				errs	+= _burst_rbg(fd, &fsamp, buf);
				break;

			case AI32SSA_BURST_SRC_EXT:

				errs	+= _burst_ext(&fsamp, buf);
				break;

			case AI32SSA_BURST_SRC_BCR:

				errs	+= _burst_bcr(&fsamp, buf);
				break;
		}

		break;
	}

	if ((errs == 0) && (quiet == 0))
	{
		gsc_label_long_comma((long) fsamp);
		printf(" S/S  (%s)\n", buf);
	}

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



/******************************************************************************
*
*	Function:	ai32ssa_fsamp_ain_report_all
*
*	Purpose:
*
*		Determine and report the sample rate for all active channels.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	Report the overall sample rate here.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int ai32ssa_fsamp_ain_report_all(int fd, int index, int verbose, s32* fsamp)
{
	int	errs	= 0;
	s32	first	= 0;
	int	i;
	s32	last	= 0;
	s32	sps;
	s32	total	= 0;

	if (verbose)
		gsc_label_index("Sample Rates", index);

	errs	+= _channels(fd, &first, &last);

	if (errs == 0)
	{
		if (verbose)
		{
			printf("\n");
			gsc_label_level_inc();
		}

		for (i = first; i <= last; i++)
		{
			errs	+= ai32ssa_fsamp_ain_report(fd, index, i, 0, &sps);
			total	+= sps;
		}

		if (verbose)
		{
			gsc_label("Overall Rate");
			gsc_label_long_comma(total);
			printf(" S/S\n");
			gsc_label_level_dec();
		}
	}

	if (fsamp)
		fsamp[0]	= total;

	return(errs);
}


