// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AI32SSA/utils/util_irq0_sel.c $
// $Rev: 43893 $
// $Date: 2018-10-30 15:16:50 -0500 (Tue, 30 Oct 2018) $

// 16AI32SSA: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ai32ssa_irq0_sel
*
*	Purpose:
*
*		Provide a visual wrapper for the AI32SSA_IOCTL_IRQ0_SEL service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		The setting to apply.
*
*		get		The current setting is recorded here, if not NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	ai32ssa_irq0_sel(int fd, int index, int verbose, s32 set, s32* get)
{
	char	buf[128];
	int		errs	= 0;
	int		ret;

	if (verbose)
		gsc_label_index("IRQ0 Selection", index);

	ret		= ai32ssa_ioctl(fd, AI32SSA_IOCTL_IRQ0_SEL, &set);
	errs	+= ret ? 1 : 0;

	switch (set)
	{
		default:

			sprintf(buf, "INVALID: %ld", (long) set);
			break;

		case AI32SSA_IRQ0_INIT_DONE:

			strcpy(buf, "Initialization Done");
			break;

		case AI32SSA_IRQ0_AUTO_CAL_DONE:

			strcpy(buf, "Auto-Calibration Done");
			break;

		case AI32SSA_IRQ0_SYNC_START:

			strcpy(buf, "Sync Start");
			break;

		case AI32SSA_IRQ0_SYNC_DONE:

			strcpy(buf, "Sync Done");
			break;

		case AI32SSA_IRQ0_BURST_START:

			strcpy(buf, "Burst Start");
			break;

		case AI32SSA_IRQ0_BURST_DONE:

			strcpy(buf, "Burst Done");
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", buf);

	if (get)
		get[0]	= set;

	return(errs);
}


