// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AI32SSA/utils/util_query.c $
// $Rev: 50230 $
// $Date: 2022-02-09 16:18:17 -0600 (Wed, 09 Feb 2022) $

// 16AI32SSA: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ai32ssa_query
*
*	Purpose:
*
*		Provide a visual wrapper for the AI32SSA_IOCTL_QUERY service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the query option to access.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	ai32ssa_query(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;
	s32			query	= set;
	int			ret;
	s32			tmp;

	if (verbose)
	{
		switch (query)
		{
			default:

				errs++;
				sprintf(buf, "Query Error (#%ld)", (long) query);
				ptr	= buf;
				break;

			case AI32SSA_QUERY_AUTO_CAL_MS:		ptr	= "Auto-Cal Duration";	break;
			case AI32SSA_QUERY_CHANNEL_MAX:		ptr	= "Max Channels";		break;
			case AI32SSA_QUERY_CHANNEL_QTY:		ptr	= "Installed Channels";	break;
			case AI32SSA_QUERY_COUNT:			ptr	= "Query Options";		break;
			case AI32SSA_QUERY_DEVICE_TYPE:		ptr	= "Device Type";		break;
			case AI32SSA_QUERY_FIFO_SIZE:		ptr	= "FIFO Size";			break;
			case AI32SSA_QUERY_FSAMP_MAX:		ptr	= "Fsamp Max";			break;
			case AI32SSA_QUERY_FSAMP_MIN:		ptr	= "Fsamp Min";			break;
			case AI32SSA_QUERY_INIT_MS:			ptr	= "Init. Duration";		break;
			case AI32SSA_QUERY_MASTER_CLOCK:	ptr	= "Master Clock";		break;
			case AI32SSA_QUERY_NRATE_MAX:		ptr	= "Nrate Max";			break;
			case AI32SSA_QUERY_NRATE_MIN:		ptr	= "Nrate Min";			break;
			case AI32SSA_QUERY_NRATE_MASK:		ptr	= "Nrate Mask";			break;
			case AI32SSA_QUERY_RATE_GEN_QTY:	ptr	= "Rate Gen Qty";		break;
			case AI32SSA_QUERY_V_RANGE_HIGH:	ptr	= "Voltage Range";		break;
		}

		gsc_label_index(ptr, index);
	}

	ret		= ai32ssa_ioctl(fd, AI32SSA_IOCTL_QUERY, &set);
	errs	+= ret ? 1 : 0;

	if (verbose)
	{
		switch (query)
		{
			default:

				errs++;
				sprintf(buf, "%ld", (long) set);
				break;

			case AI32SSA_QUERY_CHANNEL_MAX:
			case AI32SSA_QUERY_CHANNEL_QTY:
			case AI32SSA_QUERY_COUNT:
			case AI32SSA_QUERY_NRATE_MAX:
			case AI32SSA_QUERY_NRATE_MIN:
			case AI32SSA_QUERY_RATE_GEN_QTY:

				sprintf(buf, "%ld", (long) set);
				break;

			case AI32SSA_QUERY_NRATE_MASK:

				sprintf(buf, "0x%lX", (long) set);
				break;

			case AI32SSA_QUERY_MASTER_CLOCK:

				sprintf(buf, "%ld Hz", (long) set);
				break;

			case AI32SSA_QUERY_AUTO_CAL_MS:
			case AI32SSA_QUERY_INIT_MS:

				sprintf(buf, "%ld ms", (long) set);
				break;

			case AI32SSA_QUERY_FSAMP_MAX:
			case AI32SSA_QUERY_FSAMP_MIN:

				sprintf(buf, "%ld S/S", (long) set);
				break;

			case AI32SSA_QUERY_V_RANGE_HIGH:

				sprintf(buf, "%s", set ? "Supported" : "Not Supported");
				break;

			case AI32SSA_QUERY_FIFO_SIZE:

				ptr	= "";
				tmp	= set;

				if ((tmp) && ((tmp % 1024) == 0))
				{
					ptr	= "K";
					tmp	/= 1024;
				}

				sprintf(buf, "%ld%s Words", (long) tmp, ptr);
				break;

			case AI32SSA_QUERY_DEVICE_TYPE:

				if (set == GSC_DEV_TYPE_16AI32SSA)
				{
					strcpy(buf, "16AI32SSA");
				}
				else
				{
					errs++;
					sprintf(buf, "INVALID: %ld", (long) set);
				}

				break;
		}

		if (errs)
			printf("FAIL <---  (%s)\n", buf);
		else
			printf("%s\n", buf);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


