// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AI32SSA/utils/util_reg.c $
// $Rev: 51925 $
// $Date: 2022-10-20 17:37:49 -0500 (Thu, 20 Oct 2022) $

// 16AI32SSA: Utilities: source file

#include "main.h"



// macros *********************************************************************

#define	_GSC_REG(a)		"GSC " #a, AI32SSA_GSC_##a, 0, 0
#define	NAME_WIDTH		21
#define	SIZEOF_ARRAY(a)	(sizeof((a)) / sizeof((a)[0]))



//*****************************************************************************
static int _bctlr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	aim[]	=
	{
		"System",
		"Reserved",
		"Zero Test",
		"+Vref Test",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved"
	};

	static const char*	v_high[]	=
	{
		"+-2.5 Volts",
		"+-5 Volts",
		"+-10 Volts",
		"+-10 Volts"
	};

	static const char*	v_low[]	=
	{
		"+-0.625 Volts",
		"+-1.25 Volts",
		"+-2.5 Volts",
		"+-2.5 Volts"
	};

	static const char*	disabled[]	= { "Disabled",			"Enabled"		};
	static const char*	dmdma[]		= { "Threshold",		"Autonomous"	};
	static const char*	enabled[]	= { "Enabled",			"Disabled"		};
	static const char*	fail[]		= { "Fail",				"Pass"			};
	static const char*	idle[]		= { "Idle",				"Active"		};
	static const char*	intern[]	= { "Internal",			"External"		};
	static const char*	no[]		= { "No",				"Yes"			};
	static const char*	twos[]		= { "Twos Compliment",	"Offset Binary"	};

	int	errs	= 0;
	s32	high;
	int	nw		= NAME_WIDTH;

	width	+= 10;

	errs	+= ai32ssa_query(fd, -1, 0, AI32SSA_QUERY_V_RANGE_HIGH, &high);

	gsc_reg_field_show(width, nw, value, 31, 20, 1, NULL,		"Reserved"			);
	gsc_reg_field_show(width, nw, value, 19, 19, 1, dmdma,		"DMDMA Mode"		);
	gsc_reg_field_show(width, nw, value, 18, 18, 1, disabled,	"Data Packing"		);
	gsc_reg_field_show(width, nw, value, 17, 17, 1, no,			"Buffer Overflow"	);
	gsc_reg_field_show(width, nw, value, 16, 16, 1, no,			"Buffer Underflow"	);
	gsc_reg_field_show(width, nw, value, 15, 15, 1, idle,		"Initialize"		);
	gsc_reg_field_show(width, nw, value, 14, 14, 1, fail,		"Auto-Cal Status"	);
	gsc_reg_field_show(width, nw, value, 13, 13, 1, idle,		"Auto-Cal"			);
	gsc_reg_field_show(width, nw, value, 12, 12, 1, idle,		"Input Sync"		);
	gsc_reg_field_show(width, nw, value, 11, 11, 1, enabled,	"Scan Marker"		);
	gsc_reg_field_show(width, nw, value, 10,  8, 1, NULL,		"Reserved"			);
	gsc_reg_field_show(width, nw, value,  7,  7, 1, intern,		"Sync Mode"			);
	gsc_reg_field_show(width, nw, value,  6,  6, 1, twos,		"Data Format"		);

	if (high)
		gsc_reg_field_show(width, nw, value,  5,  4, 1, v_high,	"Range"				);
	else
		gsc_reg_field_show(width, nw, value,  5,  4, 1, v_low,	"Range"				);

	gsc_reg_field_show(width, nw, value,  3,  3, 1, NULL,		"Reserved"			);
	gsc_reg_field_show(width, nw, value,  2,  0, 1, aim,		"Input Mode"		);

	return(errs);
}



//*****************************************************************************
static int _icr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	irq0[]	=
	{
		"Initialization Complete",
		"Auto-Cal Completed",
		"Input Sample Initiated (Sync)",
		"Input Sample Completed (data ready)",
		"Trigger Burst Initiated (Burst Busy -> High",
		"Trigger Burst Completed (Burst Busy -> Low",
		"Reserved",
		"Reserved"
	};

	static const char*	irq1[]	=
	{
		"Idle/None",
		"Input Buffer Threshold Low->High",
		"Input Buffer Threshold High->Low",
		"Input Buffer Overflow/Underflow",
		"Reserved",
		"Reserved",
		"Reserved",
		"Reserved"
	};

	static const char*	negated[]	= { "Negated",	"Asserted"	};

	int	nw	= NAME_WIDTH;

	width	+= 10;

	gsc_reg_field_show(width, nw, value, 31,  8, 1, NULL,		"Reserved"		);
	gsc_reg_field_show(width, nw, value,  7,  7, 1, negated,	"IRQ1 Request"	);
	gsc_reg_field_show(width, nw, value,  6,  4, 1, irq1,		"IRQ1 Select"	);
	gsc_reg_field_show(width, nw, value,  3,  3, 1, negated,	"IRQ0 Request"	);
	gsc_reg_field_show(width, nw, value,  2,  0, 1, irq0,		"IRQ0 Select"	);

	return(0);
}



//*****************************************************************************
static int _idbr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	tag[]	= { "Not First",	"First"	};

	int	nw	= NAME_WIDTH;

	width	+= 10;

	gsc_reg_field_show(width, nw, value, 31, 17, 1, NULL,	"Reserved"		);
	gsc_reg_field_show(width, nw, value, 16, 16, 1, tag,	"Channel Tag"	);
	gsc_reg_field_show(width, nw, value, 15,  0, 1, NULL,	"Data"			);

	return(0);
}



//*****************************************************************************
static int _idcr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	clear[]	= { "Clear",	"Set"		};
	static const char*	idle[]	= { "Idle",		"Active"	};

	int	nw	= NAME_WIDTH;

	width	+= 10;

	gsc_reg_field_show(width, nw, value, 31, 20, 1, NULL,	"Reserved"			);
	gsc_reg_field_show(width, nw, value, 19, 19, 1, clear,	"Threshold Flag"	);
	gsc_reg_field_show(width, nw, value, 18, 18, 1, idle,	"Clear Buffer"		);
	gsc_reg_field_show(width, nw, value, 17,  0, 1, NULL,	"Threshold"			);

	return(0);
}



//*****************************************************************************
static int _rgr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	enabled[]	= { "Enabled",	"Disabled"	};

	int	nw	= NAME_WIDTH;

	width	+= 10;

	gsc_reg_field_show(width, nw, value, 31, 17, 1, NULL,		"Reserved"	);
	gsc_reg_field_show(width, nw, value, 16, 16, 1, enabled,	"Disable"	);
	gsc_reg_field_show(width, nw, value, 15,  0, 1, NULL,		"Nrate"		);

	return(0);
}



//*****************************************************************************
static int _bufsr_decode(int fd, int supported, u32 value, int width)
{
	int	nw	= NAME_WIDTH;

	width	+= 10;

	gsc_reg_field_show(width, nw, value, 31, 19, 1, NULL,	"Reserved"			);
	gsc_reg_field_show(width, nw, value, 18,  0, 1, NULL,	"Filled Locations"	);

	return(0);
}



//*****************************************************************************
static int _bsizr_decode(int fd, int supported, u32 value, int width)
{
	int	nw	= NAME_WIDTH;

	width	+= 10;

	gsc_reg_field_show(width, nw, value, 31, 19, 1, NULL,	"Reserved"		);
	gsc_reg_field_show(width, nw, value, 19,  0, 1, NULL,	"Burst Size"	);

	return(0);
}



//*****************************************************************************
static int _sscr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	burst[]	=
	{
		"Disabled",
		"Rate-B Generator",
		"Ext Sync Input (target Mode)",
		"Software (BCTLR Input Sync Bit)"
	};

	static const char*	clk_src[]	=
	{
		"Rate-A Generator",
		"Rate-B Generator",
		"External Clock (clock Target mode)",
		"Software (BCTLR Input Sync Bit)"
	};

	static const char*	channels[]	=
	{
		"Single Channel Mode",
		"2 Channels (0-1)",
		"4 Channels (0-3)",
		"8 Channels (0-7)",
		"16 Channels (0-15)",
		"31/32 Channels (0-30/31)",
		"Reserved",
		"Channel Range (x-y)"
	};

	static const char*	b_src[]		= { "Master Clock",	"Rate-A Generator"	};
	static const char*	idle[]		= { "Idle",			"Active"			};

	int	nw	= NAME_WIDTH;

	width	+= 10;

	gsc_reg_field_show(width, nw, value, 31, 18, 1, NULL,		"Reserved"				);
	gsc_reg_field_show(width, nw, value, 17, 12, 1, NULL,		"Single Channel"		);
	gsc_reg_field_show(width, nw, value, 11, 11, 1, NULL,		"Reserved"				);
	gsc_reg_field_show(width, nw, value, 10, 10, 1, b_src,		"Rate-B Gen. Source"	);
	gsc_reg_field_show(width, nw, value,  9,  8, 1, burst,		"Bursting Source"		);
	gsc_reg_field_show(width, nw, value,  7,  7, 1, idle,		"Bursting"				);
	gsc_reg_field_show(width, nw, value,  6,  5, 1, NULL,		"Reserved"				);
	gsc_reg_field_show(width, nw, value,  4,  3, 1, clk_src,	"Sample Clock Source"	);
	gsc_reg_field_show(width, nw, value,  2,  0, 1, channels,	"Active Channel"		);

	return(0);
}



//*****************************************************************************
static int _acar_decode(int fd, int supported, u32 value, int width)
{
	int	nw	= NAME_WIDTH;

	width	+= 10;

	gsc_reg_field_show(width, nw, value, 31, 16, 1, NULL,	"Reserved"		);
	gsc_reg_field_show(width, nw, value, 15,  8, 1, NULL,	"Last Channel"	);
	gsc_reg_field_show(width, nw, value,  7,  0, 1, NULL,	"First Channel"	);

	return(0);
}



//*****************************************************************************
static int _bcfgr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	channels[]	=
	{
		"31 Channels",
		"8 Channels",
		"16 Channels",
		"32 Channels"
	};

	static const char*	range[]	=
	{
		"Low (2.5/1.25/0.625 V)",
		"High (10/5/2.5 V)"
	};

	static const char*	id[]		= { "ERROR",	"16AI32SSA"	};
	static const char*	master[]	= { "40 MHz",	"Reserved"	};

	int	nw	= NAME_WIDTH;

	width	+= 10;

	gsc_reg_field_show(width, nw, value, 31, 20, 1, NULL,		"Reserved"		);
	gsc_reg_field_show(width, nw, value, 19, 19, 1, range,		"Voltage Range"	);
	gsc_reg_field_show(width, nw, value, 18, 18, 1, master,		"Master Clock"	);
	gsc_reg_field_show(width, nw, value, 17, 16, 1, channels,	"Channels"		);
	gsc_reg_field_show(width, nw, value, 15, 15, 1, id,			"Product Id"	);
	gsc_reg_field_show(width, nw, value, 14, 12, 1, NULL,		"Reserved"		);
	gsc_reg_field_show(width, nw, value, 11,  0, 0, NULL,		"Firmware Rev"	);
	value	&= 0xFFF;
	printf("%03lX\n", (long) value);

	return(0);
}



//*****************************************************************************
static int _asiocr_decode(int fd, int supported, u32 value, int width)
{
	static const char*	aux[]	=
	{
		"Inactive",
		"Active Input (Low-to-High Edge)",
		"Active Output (Positive Pulse)",
		"Reserved"
	};

	static const char*	noise[]	= { "100ns-135ns",		"1.5us-2.0us"		};
	static const char*	high[]	= { "High Pulses",		"Low Pulses"		};
	static const char*	low[]	= { "Low-to-High Edge",	"High-to-Low Edge"	};

	int	nw	= NAME_WIDTH;

	width	+= 10;

	gsc_reg_field_show(width, nw, value, 31, 11, 1, NULL,	"Reserved"			);
	gsc_reg_field_show(width, nw, value, 10, 10, 1, noise,	"Noise Supression"	);
	gsc_reg_field_show(width, nw, value,  9,  9, 1, high,	"Output Polarity"	);
	gsc_reg_field_show(width, nw, value,  8,  8, 1, low,	"Input Polarity"	);
	gsc_reg_field_show(width, nw, value,  7,  4, 1, NULL,	"Reserved"			);
	gsc_reg_field_show(width, nw, value,  3,  2, 1, aux,	"Aux 1 Mode"		);
	gsc_reg_field_show(width, nw, value,  1,  0, 1, aux,	"Aux 0 Mode"		);

	return(0);
}



//*****************************************************************************
static int _smwr_decode(int fd, int supported, u32 value, int width)
{
	int	nw	= NAME_WIDTH;

	width	+= 10;

	gsc_reg_field_show(width, nw, value, 31, 16, 1, NULL,	"Reserved"	);
	gsc_reg_field_show(width, nw, value, 15,  0, 1, NULL,	"Marker"	);

	return(0);
}



// variables ******************************************************************

static gsc_reg_def_t	_gsc[]	=
{
	{ _GSC_REG(BCTLR),		0,	_bctlr_decode,	"Board Control Register"				},
	{ _GSC_REG(ICR),		0,	_icr_decode,	"Interrupt Control Register"			},
	{ _GSC_REG(IDBR),		0,	_idbr_decode,	"Input Data Buffer Register"			},
	{ _GSC_REG(IBCR),		0,	_idcr_decode,	"Input Buffer Control Register"			},
	{ _GSC_REG(RAGR),		0,	_rgr_decode,	"Rate-A Generator Register"				},
	{ _GSC_REG(RBGR),		0,	_rgr_decode,	"Rate-B Generator Register"				},
	{ _GSC_REG(BUFSR),		0,	_bufsr_decode,	"Buffer Size Register"					},
	{ _GSC_REG(BSIZR),		0,	_bsizr_decode,	"Burst Size Register"					},
	{ _GSC_REG(SSCR),		0,	_sscr_decode,	"Scan & Sync Control Register"			},
	{ _GSC_REG(ACAR),		0,	_acar_decode,	"Active Channel Assignment Register"	},
	{ _GSC_REG(BCFGR),		0,	_bcfgr_decode,	"Board Configuration Register"			},
	{ _GSC_REG(ACVR),		0,	NULL,			"Auto Cal Values Register"				},
	{ _GSC_REG(ARWR),		0,	NULL,			"Auxiliary R/W Register"				},
	{ _GSC_REG(ASIOCR),		0,	_asiocr_decode,	"Auxiliary Sync I/O Control Register"	},
	{ _GSC_REG(SMUWR),		0,	_smwr_decode,	"Scan Marker Upper Word Register"		},
	{ _GSC_REG(SMLWR),		0,	_smwr_decode,	"Scan Marker Lower Word Register"		},

	{ NULL, 0, 0, 0, 0,		NULL	}
};



//*****************************************************************************
static const gsc_reg_def_t* _find_reg(u32 reg, const gsc_reg_def_t* list)
{
	const gsc_reg_def_t*	def	= NULL;
	int						i;

	for (i = 0; list[i].name; i++)
	{
		if (reg == list[i].reg)
		{
			def	= &list[i];
			break;
		}
	}

	return(def);
}



/******************************************************************************
*
*	Function:	ai32ssa_reg_get_def_id
*
*	Purpose:
*
*		Retrieve the register definition structure given the register id.
*
*	Arguments:
*
*		reg		The id of the register to access.
*
*	Returned:
*
*		NULL	The register id wasn't found.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* ai32ssa_reg_get_def_id(u32 reg)
{
	const gsc_reg_def_t*	def;

	def	= _find_reg(reg, _gsc);
	return(def);
}



/******************************************************************************
*
*	Function:	ai32ssa_reg_get_def_index
*
*	Purpose:
*
*		Retrieve the register definition structure based on an index.
*
*	Arguments:
*
*		index	The index of the register to access.
*
*	Returned:
*
*		NULL	The index doesn't correspond to a known register.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* ai32ssa_reg_get_def_index(int index)
{
	const gsc_reg_def_t*	def;

	if (index < 0)
		def	= NULL;
	else if (index >= (SIZEOF_ARRAY(_gsc) - 1))
		def	= NULL;
	else
		def	= &_gsc[index];

	return(def);
}



/******************************************************************************
*
*	Function:	ai32ssa_reg_get_desc
*
*	Purpose:
*
*		Retrieve the description of the specified register.
*
*	Arguments:
*
*		reg		The register whose description is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* ai32ssa_reg_get_desc(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				desc;

	def	= _find_reg(reg, _gsc);

	if (def)
		desc	= def->desc;
	else
		desc	= "UNKNOWN";

	return(desc);
}



/******************************************************************************
*
*	Function:	ai32ssa_reg_get_name
*
*	Purpose:
*
*		Retrieve the name of the specified register.
*
*	Arguments:
*
*		reg		The register whose name is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* ai32ssa_reg_get_name(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				name;

	def	= _find_reg(reg, _gsc);

	if (def)
		name	= def->name;
	else
		name	= "UNKNOWN";

	return(name);
}



/******************************************************************************
*
*	Function:	ai32ssa_reg_list
*
*	Purpose:
*
*		List the GSC registers and their values.
*
*	Arguments:
*
*		fd		The handle to access the device.
*
*		detail	List the register details?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int ai32ssa_reg_list(int fd, int detail)
{
	int	errs;

	errs	= gsc_reg_list(fd, _gsc, detail, ai32ssa_reg_read);
	return(errs);
}



