// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AI32SSA/utils/util_reg_mod.c $
// $Rev: 43893 $
// $Date: 2018-10-30 15:16:50 -0500 (Tue, 30 Oct 2018) $

// 16AI32SSA: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ai32ssa_reg_mod
*
*	Purpose:
*
*		Provide a visual wrapper for the AI32SSA_IOCTL_REG_MOD service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		reg		This is the register to access.
*
*		set		This is the value to apply via the mask.
*
*		mask	These are the bits to modify.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ai32ssa_reg_mod(int fd, int index, int verbose, u32 reg, u32 set, u32 mask)
{
	gsc_reg_t	arg;
	int			errs	= 0;
	int			ret;
	const char*	name;

	if (verbose)
		gsc_label_index("Register Mod", index);

	arg.reg		= reg;
	arg.value	= set;
	arg.mask	= mask;
	ret			= ai32ssa_ioctl(fd, AI32SSA_IOCTL_REG_MOD, &arg);
	errs		+= ret ? 1 : 0;

	if (verbose)
	{
		name	= ai32ssa_reg_get_name(reg);
		printf(	"%s  (val 0x%08lX, mask 0x%08lX, %s)\n",
				errs ? "FAIL <---" : "PASS",
				(unsigned long) set,
				(unsigned long) mask,
				name ? name : "UNKNOWN");
	}

	return(errs);
}


